/*
 *      listdialog.cpp
 *
 *      Copyright 2015 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "listdialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "engine/ircengine.h"

FXDEFMAP(ChannelsList) ChannelsListMap[]={
    FXMAPFUNC(SEL_COMMAND,  ChannelsList::ID_HEADER_CHANGE,     ChannelsList::onCmdHeader),
    FXMAPFUNC(SEL_UPDATE,   ChannelsList::ID_HEADER_CHANGE,     ChannelsList::onUpdHeader),
    FXMAPFUNC(SEL_COMMAND,  ChannelsList::ID_SORT_BY_CHANNEL,   ChannelsList::onCmdSortByChannel),
    FXMAPFUNC(SEL_COMMAND,  ChannelsList::ID_SORT_BY_MEMBERS,   ChannelsList::onCmdSortByMembers),
    FXMAPFUNC(SEL_COMMAND,  ChannelsList::ID_SORT_BY_TOPIC,     ChannelsList::onCmdSortByTopic)
};

FXIMPLEMENT(ChannelsList,FXIconList,ChannelsListMap,ARRAYNUMBER(ChannelsListMap))

ChannelsList::ChannelsList()
{}

ChannelsList::ChannelsList(FXComposite* p, FXObject* tgt, FXSelector sel, FXuint opts, FXint x, FXint y, FXint w, FXint h)
    : FXIconList(p, tgt, sel, opts, x, y, w, h)
{}

long ChannelsList::onCmdHeader(FXObject*, FXSelector, void *ptr)
{
    dxutils::debugLine(FXStringFormat("Click on header %d", (FXuint)(FXuval)ptr));
    if(((FXuint)(FXuval)ptr)<4) handle(this,FXSEL(SEL_COMMAND, (ID_SORT_BY_CHANNEL+(FXuint)(FXuval)ptr)), NULL);
    return 1;
}

long ChannelsList::onUpdHeader(FXObject *, FXSelector, void *)
{
    header->setArrowDir(0, (sortfunc==ascendingCase) ? FALSE : (sortfunc==descendingCase) ? TRUE : MAYBE);          // Channel
    header->setArrowDir(1, (sortfunc==ascendingMembers) ? FALSE : (sortfunc==descendingMembers) ? TRUE : MAYBE);    // Members
    header->setArrowDir(2, (sortfunc==ascendingTopic) ? FALSE : (sortfunc==descendingTopic) ? TRUE : MAYBE);        // Topic
    return 1;
}

long ChannelsList::onCmdSortByChannel(FXObject *, FXSelector, void *)
{
    sortfunc=(sortfunc==ascendingCase) ? descendingCase : ascendingCase;
    sortItems();
    return 1;
}

long ChannelsList::onCmdSortByMembers(FXObject *, FXSelector, void *)
{
    sortfunc=(sortfunc==ascendingMembers) ? descendingMembers : ascendingMembers;
    sortItems();
    return 1;
}

long ChannelsList::onCmdSortByTopic(FXObject *, FXSelector, void *)
{
    sortfunc=(sortfunc==ascendingTopic) ? descendingTopic : ascendingTopic;
    sortItems();
    return 1;
}

FXint ChannelsList::ascendingMembers(const FXIconItem *a, const FXIconItem *b)
{
    if(FXIntVal(dxutils::getParam(a->getText(),2,FALSE,'\t')) > FXIntVal(dxutils::getParam(b->getText(),2,FALSE,'\t'))) return 1;
    if(FXIntVal(dxutils::getParam(a->getText(),2,FALSE,'\t')) < FXIntVal(dxutils::getParam(b->getText(),2,FALSE,'\t'))) return -1;
    return compareSectionCase(a->getText().text(), b->getText().text(), 1);
}

FXint ChannelsList::descendingMembers(const FXIconItem *a, const FXIconItem *b)
{
    return -ChannelsList::ascendingMembers(a, b);
}

FXint ChannelsList::ascendingTopic(const FXIconItem *a, const FXIconItem *b)
{
    return compareSectionCase(a->getText().text(),b->getText().text(),2);
}

FXint ChannelsList::descendingTopic(const FXIconItem *a, const FXIconItem *b)
{
    return -ChannelsList::ascendingTopic(a, b);
}

FXDEFMAP(ListDialog) ListDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND,          ListDialog_JOIN,                ListDialog::onJoin),
    FXMAPFUNC(SEL_CLOSE,            0,                              ListDialog::onCancel),
    FXMAPFUNC(SEL_COMMAND,          ListDialog_CLOSE,               ListDialog::onClose),
    FXMAPFUNC(SEL_KEYPRESS,         0,                              ListDialog::onKeyPress),
    FXMAPFUNC(SEL_DOUBLECLICKED,    ListDialog_TABLE,               ListDialog::onTableDoubleClick),
    FXMAPFUNC(SEL_CHANGED,          ListDialog_SEARCH,              ListDialog::onSearch)
};

FXIMPLEMENT(ListDialog, FXDialogBox, ListDialogMap, ARRAYNUMBER(ListDialogMap))

ListDialog::ListDialog(FXMainWindow *owner, IrcEngine *engine)
        : FXDialogBox(owner, _("Channels list")+FXString(" ")+engine->getNetworkName(), DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE, 0,0,0,0, 0,0,0,0, 0,0),
          m_engine(engine)
{
    m_contents = new FXVerticalFrame(this, LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_searchframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    new FXLabel(m_searchframe, _("Search"));
    m_search = new FXTextField(m_searchframe, 25, this, ListDialog_SEARCH, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X);

    m_tableframe = new FXVerticalFrame(m_contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK);
    m_table = new ChannelsList(m_tableframe, this, ListDialog_TABLE, ICONLIST_AUTOSIZE|ICONLIST_DETAILED|ICONLIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_table->appendHeader(_("Channel"), NULL, 200);
    m_table->appendHeader(_("Members"), NULL, 75);
    m_table->appendHeader(_("Topic"), NULL, 400);
    m_channels = engine->getChannelsList();

    m_buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    m_buttonClose = new dxEXButton(m_buttonframe, _("&Close"), NULL, this, ListDialog_CLOSE, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonJoin = new dxEXButton(m_buttonframe, _("&Join"), NULL, this, ListDialog_JOIN, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);

    m_engine->clearChannelsList();

    fillChannelList();
}

ListDialog::~ListDialog()
{
}

void ListDialog::create()
{
    FXDialogBox::create();
    setWidth(700);
    setHeight(350);
}

long ListDialog::onJoin(FXObject*, FXSelector, void*)
{
    m_engine->sendJoin(m_table->getItemText(m_table->getCurrentItem()).before('\t'));
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long ListDialog::onCancel(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}

long ListDialog::onClose(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long ListDialog::onKeyPress(FXObject *sender, FXSelector sel, void *ptr)
{
    if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),NULL);
        return 1;
    }
    return 0;
}

long ListDialog::onSearch(FXObject*, FXSelector, void*)
{
    fillChannelList();
    return 1;
}

void ListDialog::fillChannelList()
{
    m_table->clearItems();
    if(m_search->getText().empty())
    {
        for(FXint i=0; i<m_channels.no(); i++)
        {
            m_table->appendItem(dxutils::getParam(m_channels[i], 1, FALSE)+TAB+dxutils::getParam(m_channels[i], 2, FALSE)+TAB+dxutils::stripColors(dxutils::getParam(m_channels[i], 3, TRUE).after(':'), TRUE));
        }
    }
    else
    {
        for(FXint i=0; i<m_channels.no(); i++)
        {
            if(dxutils::getParam(m_channels[i], 1, FALSE).find(m_search->getText()) != -1)
            {
                m_table->appendItem(dxutils::getParam(m_channels[i], 1, FALSE)+TAB+dxutils::getParam(m_channels[i], 2, FALSE)+TAB+dxutils::stripColors(dxutils::getParam(m_channels[i], 3, TRUE).after(':'), TRUE));
            }
        }
    }
    m_table->setSortFunc(FXIconList::ascendingCase);
    m_table->sortItems();
    if(m_table->getNumItems())
    {
        m_table->selectItem(0);
        m_buttonJoin->enable();
    }
    else
    {
        m_buttonJoin->disable();
    }
}

long ListDialog::onTableDoubleClick(FXObject*, FXSelector, void *)
{
    m_engine->sendJoin(m_table->getItemText(m_table->getCurrentItem()).before('\t'));
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long ListDialog::onTableHeader(FXObject*, FXSelector, void *ptr)
{
    dxutils::debugLine(FXStringFormat("Click on header %d", (FXuint)(FXuval)ptr));
    return 1;
}
