/*
 *      boatstabitem.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "boatstabitem.h"
#include "icons.h"
#include "config.h"
#include "i18n.h"
#include "dxirc.h"

#define preferences dxirc::instance()->getPreferences()
#define SPACE 5
#define LINEWIDTH 1

FXDEFMAP(BoatsTabItem) BoatsTabItemMap[] = {
    FXMAPFUNC(SEL_PAINT,                BoatsTabItem_GAMECANVAS,      BoatsTabItem::onPaint),
    FXMAPFUNC(SEL_MOTION,               BoatsTabItem_GAMECANVAS,      BoatsTabItem::onMotion),
    FXMAPFUNC(SEL_LEFTBUTTONPRESS,      BoatsTabItem_GAMECANVAS,      BoatsTabItem::onLeftClick),
    FXMAPFUNC(SEL_RIGHTBUTTONPRESS,     BoatsTabItem_GAMECANVAS,      BoatsTabItem::onRightClick),
    FXMAPFUNC(SEL_COMMAND,              BoatsTabItem_NEWGAME,         BoatsTabItem::onNewGame),
    FXMAPFUNC(SEL_COMMAND,              IrcEngine_SERVER,             BoatsTabItem::onIrcEvent)
};

FXIMPLEMENT(BoatsTabItem, dxTabItem, BoatsTabItemMap, ARRAYNUMBER(BoatsTabItemMap))

BoatsTabItem::BoatsTabItem(dxTabBook *tab, const FXString &tabtext, FXbool first, FXIcon *icon, FXuint opts,
        FXint id, IrcEngine *socket)
    : dxTabItem(tab, tabtext, icon, opts, id), m_engine(socket), m_parent(tab), m_type(BOATS), m_cell(0), m_first(first),
      m_placeBoats(TRUE), m_rotated(FALSE), m_previousX(-1), m_previousY(-1), m_cells(1), m_onFire(FALSE),
      m_ready(FALSE)
{
    m_mainframe = new FXVerticalFrame(m_parent, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_gameframe = new FXVerticalFrame(m_mainframe, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_text->reparent(m_gameframe);
    delete m_text;
    m_gamecanvas = new FXCanvas(m_gameframe, this, BoatsTabItem_GAMECANVAS, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_gamecanvas->setBackColor(FXRGB(0,0,0));

    m_newGame = new FXButton(m_mainframe, _("&New game"), NULL, this, BoatsTabItem_NEWGAME);
    m_newGame->disable();

    m_statusframe = new FXHorizontalFrame(m_mainframe, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0);
    m_status = new FXLabel(m_statusframe, _("Place boats"));

    this->setIconPosition(ICON_BEFORE_TEXT);

    clearCellsAndBoats();

    m_gamecanvas->grab();
    placeBoats();
}

BoatsTabItem::~BoatsTabItem()
{
}

void BoatsTabItem::createGeom()
{
    m_mainframe->create();
}

void BoatsTabItem::reparentTab()
{
    reparent(m_parent);
    m_mainframe->reparent(m_parent);
    recalc();
    m_mainframe->recalc();
}

void BoatsTabItem::leftGame()
{
    if(m_engine && m_engine->getConnected())
        m_engine->sendCtcp(getText(), "BOATS LEFT");
}

long BoatsTabItem::onPaint(FXObject *sender, FXSelector, void *ptr)
{
    FXEvent *ev=(FXEvent*)ptr;
    FXDCWindow dc((FXCanvas*)sender,ev);
    dc.setForeground(((FXCanvas*)sender)->getBackColor());
    dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
    m_cell = FXMIN((m_gamecanvas->getWidth()-9*SPACE)/(2*bcolumns), (m_gamecanvas->getHeight()-2*SPACE)/brows);
    FXint x1 = 0;
    FXint y1 = 0;
    for(FXint x=0; x<bcolumns; ++x)
    {
        for(FXint y=0; y<brows; ++y)
        {
            dc.setLineWidth(LINEWIDTH);
            dc.setForeground(FXRGB(168,168,168));
            dc.drawRectangle(SPACE+x1,SPACE+y1, m_cell, m_cell);
            dc.drawRectangle(6*SPACE+bcolumns*m_cell+x1,SPACE+y1, m_cell, m_cell);
            y1 += m_cell;
        }
        y1 = 0;
        x1 += m_cell;
    }
    drawCells();
    return 1;
}

long BoatsTabItem::onMotion(FXObject *, FXSelector, void *ptr)
{
    FXEvent *ev=(FXEvent*)ptr;
    if(m_placeBoats)
    {
        if(ev->win_x < SPACE || ev->win_x > SPACE+bcolumns*m_cell)
            return 1;
        if(ev->win_y < SPACE || ev->win_y > SPACE+brows*m_cell)
            return 1;
        FXint x = (ev->win_x-SPACE)/m_cell;
        FXint y = (ev->win_y-SPACE)/m_cell;
        if((m_previousX!=x || m_previousY!=y)
                && m_previousX>-1 && m_previousX<bcolumns
                && m_previousY>-1 && m_previousY<brows)
        {
            for(FXint i=0; i<m_cells; i++)
            {
                if(m_rotated)
                {
                    if(m_mycells[m_previousX][m_previousY+i].type == PLACED) m_mycells[m_previousX][m_previousY+i].type = CLEAR;
                }
                else
                {
                    if(m_mycells[m_previousX+i][m_previousY].type == PLACED) m_mycells[m_previousX+i][m_previousY].type = CLEAR;
                }
            }
        }
        if(x>-1 && x<bcolumns && y>-1 && y<brows && isRightForPlaceBoat(x, y))
        {
            m_previousX = x;
            m_previousY = y;
            for(FXint i=0; i<m_cells; i++)
            {
                if(m_rotated) m_mycells[x][y+i].type = PLACED;
                else m_mycells[x+i][y].type = PLACED;
            }
        }
        drawCells();
    }
    return 1;
}

long BoatsTabItem::onLeftClick(FXObject *, FXSelector, void *ptr)
{
    FXEvent *ev=(FXEvent*)ptr;
    if(m_placeBoats)
    {
        m_gamecanvas->grab();
        if(ev->win_x < SPACE || ev->win_x > SPACE+bcolumns*m_cell)
            return 1;
        if(ev->win_y < SPACE || ev->win_y > SPACE+brows*m_cell)
            return 1;
        FXint x = (ev->win_x-SPACE)/m_cell;
        FXint y = (ev->win_y-SPACE)/m_cell;
        if(x>-1 && x<bcolumns && y>-1 && y<brows
                && m_mycells[x][y].type == PLACED)
        {
            m_previousX = -1;
            m_previousY = -1;
            for(FXint i=0; i<m_cells; i++)
            {
                if(m_rotated) m_mycells[x][y+i].type = BOAT;
                else m_mycells[x+i][y].type = BOAT;
            }
            for(FXint i=0; i<boats; i++)
            {
                if(!m_boats[i].isValid())
                {
                    m_boats[i].x = x;
                    m_boats[i].y = y;
                    m_boats[i].cells = m_cells;
                    m_boats[i].rotated = m_rotated;
                    break;
                }
            }
        }
        drawCells();
        placeBoats();
    }
    else
    {
        if(m_engine && m_engine->getConnected() && m_onFire)
        {
            if(ev->win_x < 6*SPACE+bcolumns*m_cell || ev->win_x > 6*SPACE+2*bcolumns*m_cell)
                return 1;
            if(ev->win_y < SPACE || ev->win_y > SPACE+brows*m_cell)
                return 1;
            FXint x = (ev->win_x-6*SPACE-bcolumns*m_cell)/m_cell;
            FXint y = (ev->win_y-SPACE)/m_cell;
            if(x>-1 && x<bcolumns && y>-1 && y<brows
                    && m_hiscells[x][y].type == CLEAR)
            {
                m_hiscells[x][y].type = FIRE;
                fire(x, y);
                m_onFire = FALSE;
            }
            drawCells();
        }
    }
    return 1;
}

long BoatsTabItem::onRightClick(FXObject *, FXSelector, void *ptr)
{
    if(m_placeBoats)
    {
        FXEvent *ev=(FXEvent*)ptr;
        if(ev->win_x < SPACE || ev->win_x > SPACE+bcolumns*m_cell)
            return 1;
        if(ev->win_y < SPACE || ev->win_y > SPACE+brows*m_cell)
            return 1;
        FXint x = (ev->win_x-SPACE)/m_cell;
        FXint y = (ev->win_y-SPACE)/m_cell;
        if(x>-1 && x<bcolumns && y>-1 && y<brows && isRightForPlaceBoat(x, y))
        {
            for(FXint i=0; i<m_cells; i++)
            {
                if(m_rotated) m_mycells[x][y+i].type = CLEAR;
                else m_mycells[x+i][y].type = CLEAR;
            }
        }
        m_rotated = !m_rotated;
        if(x>-1 && x<bcolumns && y>-1 && y<brows && isRightForPlaceBoat(x, y))
        {
            for(FXint i=0; i<m_cells; i++)
            {
                if(m_rotated) m_mycells[x][y+i].type = PLACED;
                else m_mycells[x+i][y].type = PLACED;
            }
        }
        drawCells();
    }
    return 1;
}

long BoatsTabItem::onNewGame(FXObject *, FXSelector, void *)
{
    m_newGame->disable();
    clearCellsAndBoats();
    drawCells();
    m_placeBoats = TRUE;
    m_onFire = FALSE;
    if(m_engine->getConnected())
    {
        m_engine->sendCtcp(getText(), "BOATS NEWGAME");
    }
    m_status->setText(_("Place boats"));
    return 1;
}

void BoatsTabItem::placeBoats()
{
    FXint cells = 0;
    for(FXint x=0; x<bcolumns; ++x)
    {
        for(FXint y=0; y<brows; ++y)
        {
            if(m_mycells[x][y].type == BOAT)
                cells++;
        }
    }
    if(cells < 4)
    {
        m_cells = 1;
        return;
    }
    if(cells < 10)
    {
        m_cells = 2;
        return;
    }
    if(cells < 16)
    {
        m_cells = 3;
        return;
    }
    if(cells == 16)
    {
        m_cells = 4;
        return;
    }
    if(cells == 20)
    {
        m_gamecanvas->ungrab();
        m_placeBoats = FALSE;
        m_status->setText(_("Placing of boats finished"));
        if(m_engine && m_engine->getConnected())
        {
            m_engine->sendCtcp(getText(), "BOATS READY");
        }
        if(m_ready)
        {
            if(m_first)
            {
                m_status->setText(_("Fire NOW!"));
            }
            else
            {
                m_status->setText(_("Waiting for adversary fire."));
            }
        }
        return;
    }
}

FXColor BoatsTabItem::getCellColor(CELLTYPE type)
{
    switch (type) {
    case CLEAR: return FXRGB(0,0,0);
    case BOAT: return FXRGB(78,154,6);
    case WATER: return FXRGB(32,74,135);
    case HIT: return FXRGB(164,0,0);
    case PLACED: return FXRGB(78,154,6);
    case FIRE: return FXRGB(196,160,0);
    default: return FXRGB(0,0,0);
    }
}

void BoatsTabItem::drawCell(FXint x, FXint y, FXbool my)
{
    FXDCWindow dc(m_gamecanvas);
    if(my)
    {
        dc.setForeground(getCellColor(m_mycells[x][y].type));
        dc.fillRectangle(SPACE+x*m_cell+1, SPACE+y*m_cell+1, m_cell-LINEWIDTH, m_cell-LINEWIDTH);
    }
    else
    {
        dc.setForeground(getCellColor(m_hiscells[x][y].type));
        dc.fillRectangle(6*SPACE+bcolumns*m_cell+x*m_cell+1, SPACE+y*m_cell+1, m_cell-LINEWIDTH, m_cell-LINEWIDTH);
    }
}

void BoatsTabItem::drawCells()
{
    for(FXint x=0; x<bcolumns; ++x)
    {
        for(FXint y=0; y<brows; ++y)
        {
            drawCell(x, y);
            drawCell(x, y, FALSE);
        }
    }
}

FXbool BoatsTabItem::isRightForPlaceBoat(FXint x, FXint y)
{
    if(m_rotated)
    {
        if(y+m_cells-1 < brows-1 && m_mycells[x][y+m_cells].type == BOAT)
        {
            return FALSE;
        }
        for(FXint i=0; i<m_cells; i++)
        {
            if(y+i > brows-1)
            {
                return FALSE;
            }
            if(m_mycells[x][y+i].type == BOAT)
            {
                return FALSE;
            }
            if(x > 0 && m_mycells[x-1][y+i].type == BOAT)
            {
                return FALSE;
            }
            if(x < bcolumns-1 && m_mycells[x+1][y+i].type == BOAT)
            {
                return FALSE;
            }
            if(y > 0 && m_mycells[x][y-1].type == BOAT)
            {
                return FALSE;
            }
            if(y < brows-1 && m_mycells[x][y+1].type == BOAT)
            {
                return FALSE;
            }
        }
    }
    else
    {
        if(x+m_cells-1 < bcolumns-1 && m_mycells[x+m_cells][y].type == BOAT)
        {
            return FALSE;
        }
        for(FXint i=0; i<m_cells; i++)
        {
            if(x+i > bcolumns-1)
            {
                return FALSE;
            }
            if(m_mycells[x+i][y].type == BOAT)
            {
                return FALSE;
            }
            if(x > 0 && m_mycells[x-1][y].type == BOAT)
            {
                return FALSE;
            }
            if(x < bcolumns-1 && m_mycells[x+1][y].type == BOAT)
            {
                return FALSE;
            }
            if(y > 0 && m_mycells[x+i][y-1].type == BOAT)
            {
                return FALSE;
            }
            if(y < brows-1 && m_mycells[x+i][y+1].type == BOAT)
            {
                return FALSE;
            }
        }
    }
    return TRUE;
}

void BoatsTabItem::fire(FXint x, FXint y)
{
    if(m_engine && m_engine->getConnected())
    {
        m_engine->sendCtcp(getText(), FXStringFormat("BOATS %c%c FIRE", x+65, y+48));
        m_status->setText(_("Waiting for response of fire"));
    }
}

void BoatsTabItem::clearCellsAndBoats()
{
    for(FXint x=0; x<bcolumns; ++x)
    {
        for(FXint y=0; y<brows; ++y)
        {
            m_mycells[x][y].type = CLEAR;
            m_hiscells[x][y].type = CLEAR;
        }
    }
    for(FXint i=0; i<boats; ++i)
    {
        m_boats[i].x = -1;
        m_boats[i].y = -1;
    }
    m_cells = 1;
}

FXbool BoatsTabItem::boatDestroyed(FXint x, FXint y)
{
    for(FXint i=0; i<boats; ++i)
    {
        if(m_boats[i].constainsCell(x, y))
        {
            if(m_boats[i].rotated)
            {

                for(FXint y1=m_boats[i].y; y1<brows && y1<m_boats[i].y+m_boats[i].cells; ++y1)
                {
                    if(m_mycells[x][y1].type != HIT)
                        return FALSE;
                }
            }
            else
            {
                for(FXint x1=m_boats[i].x; x1<bcolumns && x1<m_boats[i].x+m_boats[i].cells; ++x1)
                {
                    if(m_mycells[x1][y].type != HIT)
                        return FALSE;
                }
            }
            return TRUE;
        }
    }
    return FALSE;
}

FXbool BoatsTabItem::gameOver()
{
    for(FXint i=0; i<boats; ++i)
    {
        if(m_boats[i].rotated)
        {

            for(FXint y1=m_boats[i].y; y1<brows && y1<m_boats[i].y+m_boats[i].cells; ++y1)
            {
                if(m_mycells[m_boats[i].x][y1].type != HIT)
                    return FALSE;
            }
        }
        else
        {
            for(FXint x1=m_boats[i].x; x1<bcolumns && x1<m_boats[i].x+m_boats[i].cells; ++x1)
            {
                if(m_mycells[x1][m_boats[i].y].type != HIT)
                    return FALSE;
            }
        }
    }
    return TRUE;
}

long BoatsTabItem::onIrcEvent(FXObject *, FXSelector, void *data)
{
    IrcEvent ev = *(IrcEvent *)data;
    if(ev.eventType == IRC_NICK)
    {
        if(ev.param1 == getText())
        {
            this->setText(ev.param2);
        }
        return 1;
    }
    if(ev.eventType == IRC_QUIT && getText() == ev.param1)
    {
        m_status->setText(FXStringFormat(_("%s has quit"), ev.param1.text()));
        m_placeBoats = FALSE;
        m_ready = FALSE;
        m_onFire = FALSE;
        return 1;
    }
    if(ev.eventType == IRC_401 && getText() == ev.param1)
    {
        m_status->setText(FXStringFormat(_("%s isn't online"), ev.param1.text()));
        m_placeBoats = FALSE;
        m_ready = FALSE;
        m_onFire = FALSE;
        return 1;
    }
    if(ev.eventType == IRC_BOATS && !comparecase(ev.param1, getText()))
    {
        //position
        if(ev.param2.length()!=2)
            return 1;
        if(ev.param2[0]<65 || ev.param2[0]>74)
            return 1;
        if(ev.param2[1]<48 || ev.param2[1]>57)
            return 1;
        FXint x = ev.param2[0]-65;
        FXint y = ev.param2[1]-48;
        //redraw cell
        if(ev.param3 == "FIRE")
        {
            if(m_mycells[x][y].type == BOAT)
            {
                m_mycells[x][y].type = HIT;
                m_engine->sendCtcp(getText(), FXStringFormat("BOATS %s HIT", ev.param2.text()));
                if(boatDestroyed(x, y))
                    m_engine->sendCtcp(getText(), "BOATS DESTROYED");
                if(gameOver())
                {
                    m_ready = FALSE;
                    m_onFire = FALSE;
                    m_status->setText(_("You LOST."));
                    m_newGame->enable();
                    m_engine->sendCtcp(getText(), "BOATS WIN");
                    drawCell(x, y);
                    return 1;
                }
            }
            else
            {
                m_mycells[x][y].type = WATER;
                m_engine->sendCtcp(getText(), FXStringFormat("BOATS %s FAIL", ev.param2.text()));
            }
            drawCell(x, y);
            m_status->setText(_("Adversary fired. You fire NOW!"));
            m_onFire = TRUE;
            return 1;
        }
        if(ev.param3 == "HIT")
        {
            m_hiscells[x][y].type = HIT;
            drawCell(x, y, FALSE);
            m_status->setText(_("Waiting for adversary fire."));
            return 1;
        }
        if(ev.param3 == "FAIL")
        {
            m_hiscells[x][y].type = WATER;
            drawCell(x, y, FALSE);
            m_status->setText(_("Waiting for adversary fire."));
            return 1;
        }
        return 1;
    }
    if(ev.eventType == IRC_BOATSREADY && !comparecase(ev.param1, getText()))
    {
        m_ready = TRUE;
        if(m_first)
        {
            m_onFire = TRUE;
            if(!m_placeBoats) m_status->setText(_("Fire NOW!"));
        }
        else
        {
            m_onFire = FALSE;
            if(!m_placeBoats) m_status->setText(_("Waiting for adversary fire."));
        }
        return 1;
    }
    if(ev.eventType == IRC_BOATSLEFT && !comparecase(ev.param1, getText()))
    {
        m_placeBoats = FALSE;
        m_ready = FALSE;
        m_onFire = FALSE;
        m_status->setText(_("Adversary left game."));
        return 1;
    }
    if(ev.eventType == IRC_BOATSDESTROYED && !comparecase(ev.param1, getText()))
    {
        m_status->setText(_("Boat was destroyed. Waiting for adversary fire."));
        return 1;
    }
    if(ev.eventType == IRC_BOATSWIN && !comparecase(ev.param1, getText()))
    {
        m_ready = FALSE;
        m_onFire = FALSE;
        m_status->setText(_("You WIN."));
        m_newGame->enable();
        return 1;
    }
    if(ev.eventType == IRC_BOATSNEWGAME && !comparecase(ev.param1, getText()))
    {
        clearCellsAndBoats();
        drawCells();
        m_placeBoats = TRUE;
        m_ready = FALSE;
        m_onFire = FALSE;
        m_status->setText(_("Place boats"));
        m_newGame->disable();
        return 1;
    }
    return 1;
}
