/*
 *      irctabitem.cpp
 *
 *      Copyright 2008 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>
#endif
#include <string>
#include "irctabitem.h"
#include "tetristabitem.h"
#include "icons.h"
#include "config.h"
#include "i18n.h"
#include "dxirc.h"

#define ICON_SPACING             4    // Spacing between icon and label
#define SIDE_SPACING             6    // Left or right spacing between items
#define LINE_SPACING             4    // Line spacing between items

#define preferences dxirc::instance()->getPreferences()

//Get away color
FXColor makeAwayColor(FXColor clr)
{
    FXuint r,g,b;
    r=FXREDVAL(clr);
    g=FXGREENVAL(clr);
    b=FXBLUEVAL(clr);
    if(r==g && r==b && !r)
    {
        r=255;
        g=255;
        b=255;
    }
    r=(50*r)/100;
    g=(50*g)/100;
    b=(50*b)/100;
    return FXRGB(r,g,b);
}

//Sort by UserMode and nick
FXint sortNick(const FXListItem *a, const FXListItem *b)
{
    return comparecase((FXchar)((NickListItem*)a)->getUserMode()+a->getText(), (FXchar)((NickListItem*)b)->getUserMode()+b->getText());
}

FXDEFMAP(DccSendDialog) DccSendDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND,  DccSendDialog_FILE,     DccSendDialog::onFile),
    FXMAPFUNC(SEL_COMMAND,  DccSendDialog_SEND,     DccSendDialog::onSend),
    FXMAPFUNC(SEL_COMMAND,  DccSendDialog_CANCEL,   DccSendDialog::onCancel),
    FXMAPFUNC(SEL_CLOSE,    0,                      DccSendDialog::onCancel),
    FXMAPFUNC(SEL_KEYPRESS, 0,                      DccSendDialog::onKeyPress)
};

FXIMPLEMENT(DccSendDialog, FXDialogBox, DccSendDialogMap, ARRAYNUMBER(DccSendDialogMap))

DccSendDialog::DccSendDialog(FXMainWindow* owner, FXString nick)
        : FXDialogBox(owner, FXStringFormat(_("Send file to %s"), nick.text()), DECOR_RESIZE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_mainFrame = new FXVerticalFrame(this, LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_fileFrame = new FXHorizontalFrame(m_mainFrame, LAYOUT_FILL_X);
    new FXLabel(m_fileFrame, _("File:"));
    m_fileText = new FXTextField(m_fileFrame, 25, NULL, 0, TEXTFIELD_READONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X);
    m_buttonFile = new dxEXButton(m_fileFrame, "...", NULL, this, DccSendDialog_FILE, FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X, 0,0,0,0, 10,10,2,2);

    m_passiveFrame = new FXHorizontalFrame(m_mainFrame, LAYOUT_FILL_X);
    m_checkPassive = new FXCheckButton(m_passiveFrame, _("Send passive"), NULL, 0);

    m_buttonFrame = new FXHorizontalFrame(m_mainFrame, LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
    m_buttonCancel = new dxEXButton(m_buttonFrame, _("&Cancel"), NULL, this, DccSendDialog_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonSend = new dxEXButton(m_buttonFrame, _("&Send file"), NULL, this, DccSendDialog_SEND, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
}

DccSendDialog::~DccSendDialog()
{
}

FXuint DccSendDialog::execute(FXuint placement)
{
    create();
    show(placement);
    getApp()->refresh();
    dxEXFileDialog dialog(this, _("Select file"));
    if(dialog.execute())
    {
        m_fileText->setText(dialog.getFilename());
    }
    return getApp()->runModalFor(this);
}

long DccSendDialog::onFile(FXObject*, FXSelector, void*)
{
    dxEXFileDialog dialog(this, _("Select file"));
    if(dialog.execute())
    {
        m_fileText->setText(dialog.getFilename());
    }
    return 1;
}

long DccSendDialog::onSend(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long DccSendDialog::onCancel(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}

long DccSendDialog::onKeyPress(FXObject *sender, FXSelector sel, void *ptr)
{
    if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        handle(this,FXSEL(SEL_COMMAND,DccSendDialog_CANCEL),NULL);
        return 1;
    }
    return 0;
}

FXDEFMAP(NickList) NickListMap [] = {
    FXMAPFUNC(SEL_QUERY_TIP, 0, NickList::onQueryTip)
};

FXIMPLEMENT(NickList, FXList, NickListMap, ARRAYNUMBER(NickListMap))

NickList::NickList(FXComposite* p, IrcTabItem* tgt, FXSelector sel, FXuint opts, FXint x, FXint y, FXint w, FXint h)
        : FXList(p, tgt, sel, opts, x, y, w, h), m_parent(tgt)
{
}

long NickList::onQueryTip(FXObject *sender, FXSelector sel, void *ptr)
{
    if(FXWindow::onQueryTip(sender, sel, ptr)) return 1;
    if((flags&FLAG_TIP) && !(options&LIST_AUTOSELECT) && (0<=cursor)) // No tip when autoselect!
    {
        FXString string = items[cursor]->getText();
        string.append('\n');
        NickInfo nick = m_parent->m_engine->getNickInfo(items[cursor]->getText());
        string.append(FXStringFormat(_("User: %s@%s\n"), nick.user.text(), nick.host.text()));
        string.append(FXStringFormat(_("Realname: %s"), nick.real.text()));
        sender->handle(this, FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE), (void*)&string);
        return 1;
    }
    return 0;
}

FXIMPLEMENT(NickListItem, FXListItem, NULL, 0)

// Draw item
void NickListItem::draw(const FXList* list, FXDC& dc, FXint xx, FXint yy, FXint ww, FXint hh)
{
    register FXFont *font=list->getFont();
    register FXint ih=0,th=0;
    if(icon) ih=icon->getHeight();
    if(!label.empty()) th=font->getFontHeight();
    dc.setForeground(list->getBackColor());     // FIXME maybe paint background in onPaint?
    dc.fillRectangle(xx,yy,ww,hh);
    if(isSelected())
    {
        dc.drawFocusRectangle(xx+1,yy+1,ww-2,hh-2);
    }
    xx+=SIDE_SPACING/2;
    if(icon)
    {
        dc.drawIcon(icon,xx,yy+(hh-ih)/2);
        xx+=ICON_SPACING+icon->getWidth();
    }
    else
    {
        FXString mode = "";
        switch(m_mode){
            case ADMIN: mode = "!";break;
            case OWNER: mode = "*";break;
            case OP: mode = "@";break;
            case HALFOP: mode = "%";break;
            case VOICE: mode= "+";break;
            case NONE: mode = " ";break;
        }
        dc.setFont(font);
        dc.setForeground(list->getTextColor());
        dc.drawText(xx,yy+(hh-th)/2+font->getFontAscent(),mode);
        xx+=ICON_SPACING+font->getTextWidth("@");
    }
    if(!label.empty())
    {
        dc.setFont(font);
        if(m_away)
            dc.setForeground(makeAwayColor(list->getTextColor()));
        else
            dc.setForeground(list->getTextColor());
        dc.drawText(xx,yy+(hh-th)/2+font->getFontAscent(),label);
    }
}

// Get width of item
FXint NickListItem::getWidth(const FXList* list) const
{
    register FXFont *font=list->getFont();
    register FXint w=0;
    if(icon)
    {
        w=icon->getWidth();
    }
    else
    {
        w=font->getTextWidth("@");
    }
    if(!label.empty())
    {
        if(w) w+=ICON_SPACING;
        w+=font->getTextWidth(label.text(),label.length());
    }
    return SIDE_SPACING+w;
}

void NickListItem::setUserMode(UserMode mode)
{
    m_mode = mode;
    m_parent->recalc();
    if(icon)
    {
        switch(m_mode){
            case ADMIN: setIcon(m_away? ICO_IRCAWAYADMIN:ICO_IRCADMIN);break;
            case OWNER: setIcon(m_away? ICO_IRCAWAYOWNER:ICO_IRCOWNER);break;
            case OP: setIcon(m_away? ICO_IRCAWAYOP:ICO_IRCOP);break;
            case HALFOP: setIcon(m_away? ICO_IRCAWAYHALFOP:ICO_IRCHALFOP);break;
            case VOICE: setIcon(m_away? ICO_IRCAWAYVOICE:ICO_IRCVOICE);break;
            case NONE: setIcon(m_away? ICO_IRCAWAYNORMAL:ICO_IRCNORMAL);break;
        }
    }
}

void NickListItem::changeAway(FXbool away)
{
    if(m_away == away) return;
    m_away = away;
    m_parent->recalc();
    if(icon)
    {
        switch(m_mode){
            case ADMIN: setIcon(m_away? ICO_IRCAWAYADMIN:ICO_IRCADMIN);break;
            case OWNER: setIcon(m_away? ICO_IRCAWAYOWNER:ICO_IRCOWNER);break;
            case OP: setIcon(m_away? ICO_IRCAWAYOP:ICO_IRCOP);break;
            case HALFOP: setIcon(m_away? ICO_IRCAWAYHALFOP:ICO_IRCHALFOP);break;
            case VOICE: setIcon(m_away? ICO_IRCAWAYVOICE:ICO_IRCVOICE);break;
            case NONE: setIcon(m_away? ICO_IRCAWAYNORMAL:ICO_IRCNORMAL);break;
        }
    }
}

//Clear icon, usefull for nick icon theme change
void NickListItem::clearIcon()
{
    if(icon)
        setIcon(NULL);
}

//Apply new theme icon, usefull for nick icon theme change
void NickListItem::restoreIcon()
{
    m_parent->recalc();
    switch(m_mode){
        case ADMIN: setIcon(m_away? ICO_IRCAWAYADMIN:ICO_IRCADMIN);break;
        case OWNER: setIcon(m_away? ICO_IRCAWAYOWNER:ICO_IRCOWNER);break;
        case OP: setIcon(m_away? ICO_IRCAWAYOP:ICO_IRCOP);break;
        case HALFOP: setIcon(m_away? ICO_IRCAWAYHALFOP:ICO_IRCHALFOP);break;
        case VOICE: setIcon(m_away? ICO_IRCAWAYVOICE:ICO_IRCVOICE);break;
        case NONE: setIcon(m_away? ICO_IRCAWAYNORMAL:ICO_IRCNORMAL);break;
    }
}

FXDEFMAP(IrcTabItem) IrcTabItemMap[] = {
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_COMMANDLINE,     IrcTabItem::onCommandline),
    FXMAPFUNC(SEL_KEYPRESS,             IrcTabItem_COMMANDLINE,     IrcTabItem::onKeyPress),
    FXMAPFUNC(SEL_COMMAND,              IrcEngine_SERVER,           IrcTabItem::onIrcEvent),
    FXMAPFUNC(SEL_TIMEOUT,              IrcTabItem_PTIME,           IrcTabItem::onPipeTimeout),
    FXMAPFUNC(SEL_TIMEOUT,              IrcTabItem_ETIME,           IrcTabItem::onEggTimeout),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,   IrcTabItem_USERS,           IrcTabItem::onRightMouse),
    FXMAPFUNC(SEL_DOUBLECLICKED,        IrcTabItem_USERS,           IrcTabItem::onDoubleclick),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_NEWQUERY,        IrcTabItem::onNewQuery),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_WHOIS,           IrcTabItem::onWhois),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_DCCCHAT,         IrcTabItem::onDccChat),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_DCCSEND,         IrcTabItem::onDccSend),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_OP,              IrcTabItem::onOp),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_DEOP,            IrcTabItem::onDeop),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_VOICE,           IrcTabItem::onVoice),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_DEVOICE,         IrcTabItem::onDevoice),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_KICK,            IrcTabItem::onKick),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_BAN,             IrcTabItem::onBan),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_KICKBAN,         IrcTabItem::onKickban),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_IGNORE,          IrcTabItem::onIgnore),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_TOPIC,           IrcTabItem::onTopic),
    FXMAPFUNC(SEL_LINK,                 IrcTabItem_TOPIC,           IrcTabItem::onTopicLink),
    FXMAPFUNC(SEL_COMMAND,              dxPipe::ID_PIPE,            IrcTabItem::onPipe),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_AWAY,            IrcTabItem::onSetAway),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_DEAWAY,          IrcTabItem::onRemoveAway),
    FXMAPFUNC(SEL_COMMAND,              IrcTabItem_SPELL,           IrcTabItem::onSpellLang),
    FXMAPFUNC(SEL_COMMAND,              LogThread_LIST,             IrcTabItem::onListLogThread),
    FXMAPFUNC(SEL_COMMAND,              LogThread_LINES,            IrcTabItem::onLinesLogThread)
};

FXIMPLEMENT(IrcTabItem, dxTabItem, IrcTabItemMap, ARRAYNUMBER(IrcTabItemMap))

IrcTabItem::IrcTabItem(dxTabBook *tab, const FXString &tabtext, FXIcon *icon, FXuint opts,
        FXint id, TYPE type, IrcEngine *socket, FXFont *font, FXbool useNickIcon)
    : dxTabItem(tab, tabtext, icon, opts, id), m_engine(socket), m_parent(tab), m_type(type),
        m_useNickIcon(useNickIcon), m_logstream(NULL)
{
    m_currentPosition = 0;
    m_historyMax = 25;
    m_numberUsers = 0;
    m_maxLen = 460;
    m_iamOp = FALSE;
    m_topic = _("No topic is set");
    m_editableTopic = TRUE;
    m_pipe = NULL;
    m_sendPipe = FALSE;
    m_scriptHasAll = FALSE;
    m_scriptHasOwnMsg = FALSE;
    m_loadLog = FALSE;

    if(m_type == CHANNEL && m_engine && m_engine->getConnected())
    {
        m_engine->sendMode(getText());
    }

    m_mainframe = new FXVerticalFrame(m_parent, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_splitter = new FXSplitter(m_mainframe, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_REVERSED|SPLITTER_TRACKING);

    m_textframe = new FXVerticalFrame(m_splitter, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_topicline = new dxTextField(m_textframe, 50, this, IrcTabItem_TOPIC, FRAME_SUNKEN|TEXTFIELD_ENTER_ONLY|JUSTIFY_LEFT|LAYOUT_FILL_X);
    m_topicline->setFont(font);
    m_topicline->setLinkColor(preferences.m_colors.link);
    m_topicline->setText(m_topic);
    m_topicline->setUseLink(TRUE);
    m_topicline->setTopicline(TRUE);
    if(m_type != CHANNEL)
    {
        m_topicline->hide();
    }
    m_text->reparent(m_textframe);
    m_text->setFont(font);
    m_text->setSelTextColor(getApp()->getSelforeColor());
    m_text->setSelBackColor(getApp()->getSelbackColor());

    m_usersframe = new FXVerticalFrame(m_splitter, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH);
    m_users = new NickList(m_usersframe, this, IrcTabItem_USERS, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_users->setSortFunc(preferences.m_sortUserMode ? sortNick : FXList::ascendingCase);
    m_users->setScrollStyle(HSCROLLING_OFF);
    if(preferences.m_sameList) m_users->setFont(font);
    if(m_type != CHANNEL || !preferences.m_usersShown)
    {
        m_usersframe->hide();
        m_users->hide();
    }

    m_commandframe = new FXHorizontalFrame(m_mainframe, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0);
    m_commandline = new dxTextField(m_commandframe, 25, this, IrcTabItem_COMMANDLINE, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_BOTTOM, 0, 0, 0, 0, 1, 1, 1, 1);
    if(preferences.m_sameCmd) m_commandline->setFont(font);
    m_spellLangs = new FXComboBox(m_commandframe, 6, this, IrcTabItem_SPELL, COMBOBOX_STATIC);
    m_spellLangs->setTipText(_("Spellchecking language list"));
    m_spellLangs->hide();
    if(preferences.m_sameCmd) m_spellLangs->setFont(font);
    if(preferences.m_useSpell && (m_type==CHANNEL || m_type==QUERY) && dxUtils.getAvailableSpellLangsNum())
    {
        dxStringArray langs = dxUtils.getAvailableSpellLangs();
        FXString lang = dxUtils.getChannelLang(getText());
        for(FXint i=0; i<langs.no(); i++)
        {
            m_spellLangs->appendItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentItem(i);;
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(TRUE);
        m_commandline->setLanguage(lang);
        m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),lang.text()));
    }

    dxHiliteStyle style = {preferences.m_colors.text,preferences.m_colors.back,getApp()->getSelforeColor(),getApp()->getSelbackColor(),0,FALSE};
    for(int i=0; i<19; i++)
    {
        m_textStyleList.append(style);
    }
    //gray text - user commands
    m_textStyleList[0].normalForeColor = preferences.m_colors.user;
    //orange text - Actions
    m_textStyleList[1].normalForeColor = preferences.m_colors.action;
    //blue text - Notice
    m_textStyleList[2].normalForeColor = preferences.m_colors.notice;
    //red text - Errors
    m_textStyleList[3].normalForeColor = preferences.m_colors.error;
    //bold style
    m_textStyleList[4].style = FXText::STYLE_BOLD;
    //underline style
    m_textStyleList[5].style = FXText::STYLE_UNDERLINE;
    //bold & underline
    m_textStyleList[6].style = FXText::STYLE_UNDERLINE;
    m_textStyleList[6].style ^=FXText::STYLE_BOLD;
    //highlight text
    m_textStyleList[7].normalForeColor = preferences.m_colors.hilight;
    //link style
    m_textStyleList[8].normalForeColor = preferences.m_colors.link;
    m_textStyleList[8].link = TRUE;
    //mymsg style
    m_textStyleList[9].normalForeColor = preferences.m_colors.mymsg;
    //log line style
    m_textStyleList[10].normalForeColor = preferences.m_colors.log;
    //next styles for colored nicks
    m_textStyleList[11].normalForeColor = FXRGB(196, 160, 0);
    m_textStyleList[12].normalForeColor = FXRGB(206, 92, 0);
    m_textStyleList[13].normalForeColor = FXRGB(143, 89, 2);
    m_textStyleList[14].normalForeColor = FXRGB(78, 154, 6);
    m_textStyleList[15].normalForeColor = FXRGB(32, 74, 135);
    m_textStyleList[16].normalForeColor = FXRGB(117, 80, 123);
    m_textStyleList[17].normalForeColor = FXRGB(164, 0, 0);
    m_textStyleList[18].normalForeColor = FXRGB(85, 87, 83);

    //text->setStyled(TRUE);
    m_text->setHiliteStyles(m_textStyleList.data());

    m_text->setBackColor(preferences.m_colors.back);
    m_commandline->setBackColor(preferences.m_colors.back);
    m_topicline->setBackColor(preferences.m_colors.back);
    m_users->setBackColor(preferences.m_colors.back);
    m_text->setTextColor(preferences.m_colors.text);
    m_commandline->setTextColor(preferences.m_colors.text);
    m_commandline->setCursorColor(preferences.m_colors.text);
    m_topicline->setTextColor(preferences.m_colors.text);
    m_topicline->setCursorColor(preferences.m_colors.text);
    m_users->setTextColor(preferences.m_colors.text);
    m_spellLangs->setBackColor(preferences.m_colors.back);
    m_spellLangs->setTextColor(preferences.m_colors.text);

    this->setIconPosition(ICON_BEFORE_TEXT);
    
    if(m_engine) m_logThread = new LogThread(this, getApp(), preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText());
}

IrcTabItem::~IrcTabItem()
{
    this->stopLogging();
    if(m_pipe) m_pipe->closePipe();
    m_pipeStrings.clear();
    getApp()->removeTimeout(this, IrcTabItem_PTIME);
}

void IrcTabItem::createGeom()
{
    m_mainframe->create();
    loadLogLines();
    m_commandline->setFocus();
}

void IrcTabItem::clearChat()
{
    m_textStyleList.no(19);
    m_text->setHiliteStyles(m_textStyleList.data());
    m_text->clearText();
}

//usefull for set tab current
void IrcTabItem::makeLastRowVisible()
{
    m_text->makeLastRowVisible(TRUE);
}

FXString IrcTabItem::getSpellLang()
{
#ifdef HAVE_ENCHANT
    if(m_spellLangs->getNumItems()) return m_spellLangs->getItemText(m_spellLangs->getCurrentItem());
#endif
    return "";
}

void IrcTabItem::reparentTab()
{
    reparent(m_parent);
    m_mainframe->reparent(m_parent);
    recalc();
    m_mainframe->recalc();
}

void IrcTabItem::hideUsers()
{
    if(m_type == CHANNEL)
    {
        m_usersframe->hide();
        m_users->hide();
        m_splitter->setSplit(1, 0);
    }
}

void IrcTabItem::showUsers()
{
    if(m_type == CHANNEL)
    {
        m_usersframe->show();
        m_users->show();
        m_splitter->recalc();
    }
}

void IrcTabItem::setType(const TYPE &typ, const FXString &tabtext)
{
    if(typ == CHANNEL)
    {
        if(preferences.m_usersShown) m_usersframe->show();
        if(preferences.m_usersShown) m_users->show();
        m_topicline->show();
        m_topicline->setText(m_topic);
        m_splitter->recalc();
        setText(tabtext);
        if(m_engine->getConnected()) m_engine->sendMode(getText());
        m_type = typ;
    }
    else if(typ == SERVER || typ == QUERY)
    {
        m_usersframe->hide();
        m_users->hide();
        m_topicline->setText("");
        m_topicline->hide();
        m_topic = _("No topic is set");
        setText(tabtext);
        m_splitter->setSplit(1, 0);
        if(m_type == CHANNEL)
        {
            m_users->clearItems();
            m_numberUsers = 0;
        }
        m_type = typ;
    }
    this->stopLogging();
    if(m_type == SERVER) this->setIcon(ICO_SERVER);
    else if(m_type == CHANNEL) this->setIcon(ICO_CHANNEL);
    else this->setIcon(ICO_QUERY);
    if(m_type==CHANNEL || m_type==QUERY)
        loadLogLines();
    if(preferences.m_useSpell && (m_type==CHANNEL || m_type==QUERY) && dxUtils.getAvailableSpellLangsNum())
    {
        dxStringArray langs = dxUtils.getAvailableSpellLangs();
        FXString lang = dxUtils.getChannelLang(getText());
        for(FXint i=0; i<langs.no(); i++)
        {
            m_spellLangs->appendItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentItem(i);
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(TRUE);
        m_commandline->setLanguage(lang);
        m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),lang.text()));
    }
    else
    {
        m_commandline->setUseSpell(FALSE);
        m_commandline->setTipText("");
        m_spellLangs->hide();
    }
    m_commandframe->recalc();
}

void IrcTabItem::preferencesUpdated()
{
    setTextForeColor(preferences.m_colors.text);
    setTextBackColor(preferences.m_colors.back);
    setUserColor(preferences.m_colors.user);
    setActionsColor(preferences.m_colors.action);
    setNoticeColor(preferences.m_colors.notice);
    setErrorColor(preferences.m_colors.error);
    setHilightColor(preferences.m_colors.hilight);
    setLinkColor(preferences.m_colors.link);
    setMymsgColor(preferences.m_colors.mymsg);
    setLogColor(preferences.m_colors.log);
    stopLogging();
    if(preferences.m_useSpell && (m_type==CHANNEL || m_type==QUERY) && dxUtils.getAvailableSpellLangsNum())
    {
        dxStringArray langs = dxUtils.getAvailableSpellLangs();
        FXString lang = dxUtils.getChannelLang(getText());
        for(FXint i=0; i<langs.no(); i++)
        {
            m_spellLangs->appendItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentItem(i);;
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(TRUE);
        m_commandline->setLanguage(lang);
        m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),lang.text()));
    }
    else
    {
        m_commandline->setUseSpell(FALSE);
        m_commandline->setTipText("");
        m_spellLangs->hide();
    }
    m_users->setSortFunc(preferences.m_sortUserMode ? sortNick : FXList::ascendingCase);
    m_users->sortItems();
    m_commandframe->recalc();
}

void IrcTabItem::setTextBackColor(FXColor clr)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        m_textStyleList[i].normalBackColor = clr;
    }
    m_text->setBackColor(clr);
    m_commandline->setBackColor(clr);
    m_topicline->setBackColor(clr);
    m_users->setBackColor(clr);
    m_spellLangs->setBackColor(clr);
}

void IrcTabItem::setTextForeColor(FXColor clr)
{
    m_textStyleList[4].normalForeColor = clr;
    m_textStyleList[5].normalForeColor = clr;
    m_textStyleList[6].normalForeColor = clr;
    m_text->setTextColor(clr);
    m_commandline->setTextColor(clr);
    m_commandline->setCursorColor(clr);
    m_topicline->setTextColor(clr);
    m_topicline->setCursorColor(clr);
    m_users->setTextColor(clr);
    m_spellLangs->setTextColor(clr);
}

void IrcTabItem::setUserColor(FXColor clr)
{
    m_textStyleList[0].normalForeColor = clr;
}

void IrcTabItem::setActionsColor(FXColor clr)
{
    m_textStyleList[1].normalForeColor = clr;
}

void IrcTabItem::setNoticeColor(FXColor clr)
{
    m_textStyleList[2].normalForeColor = clr;
}

void IrcTabItem::setErrorColor(FXColor clr)
{
    m_textStyleList[3].normalForeColor = clr;
}

void IrcTabItem::setHilightColor(FXColor clr)
{
    m_textStyleList[7].normalForeColor = clr;
}

void IrcTabItem::setLinkColor(FXColor clr)
{
    m_textStyleList[8].normalForeColor = clr;
    m_topicline->setLinkColor(clr);
}

void IrcTabItem::setMymsgColor(FXColor clr)
{
    m_textStyleList[9].normalForeColor = clr;
}

void IrcTabItem::setLogColor(FXColor clr)
{
    m_textStyleList[10].normalForeColor = clr;
}

void IrcTabItem::setIrcFont(FXFont *fnt)
{
    m_text->setFont(fnt);
    m_text->recalc();
    m_topicline->setFont(fnt);
    m_topicline->recalc();
    if(preferences.m_sameCmd)
    {
        m_commandline->setFont(fnt);
        m_commandline->recalc();
        m_spellLangs->setFont(fnt);
        m_spellLangs->recalc();
    }
    else
    {
        m_commandline->setFont(getApp()->getNormalFont());
        m_commandline->recalc();
        m_spellLangs->setFont(getApp()->getNormalFont());
        m_spellLangs->recalc();
    }
    if(preferences.m_sameList)
    {
        m_users->setFont(fnt);
        m_users->recalc();
    }
    else
    {
        m_users->setFont(getApp()->getNormalFont());
        m_users->recalc();
    }
}

void IrcTabItem::setSmileys(FXbool smiley, dxSmileyArray nsmileys)
{
    m_text->setSmileys(smiley, nsmileys);
}

void IrcTabItem::showSpellComboUpdated()
{
    if(m_type == CHANNEL || m_type == QUERY)
    {
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        else m_spellLangs->hide();
        m_commandframe->recalc();
    }
}

void IrcTabItem::removeSmileys()
{
    m_text->removeSmileys();
}

//if highlight==TRUE, highlight tab
void IrcTabItem::appendText(FXString msg, FXbool highlight, FXbool logLine)
{
    appendIrcText(msg, 0, FALSE, logLine);
    if(highlight && preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
    {
        if(highlightNeeded(msg))
        {
            this->setTextColor(preferences.m_highlightColor);
            if(m_type == CHANNEL) this->setIcon(ICO_CHANNELNEWMSG);
        }
        else this->setTextColor(preferences.m_unreadColor);
        if(m_type == QUERY) this->setIcon(ICO_QUERYNEWMSG);
    }
}

void IrcTabItem::appendIrcText(FXString msg, FXTime time, FXbool disableStrip, FXbool logLine)
{
    if(!time) time = FXSystem::now();
    if(m_type != OTHER) m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    if(preferences.m_stripColors && !disableStrip) msg = dxutils::stripColors(msg, FALSE);
    appendLinkText(msg, 0);
    if(logLine) this->logLine(msg, time);
}

void IrcTabItem::appendIrcNickText(FXString nick, FXString msg, FXint style, FXTime time, FXbool logLine)
{
    if(!time) time = FXSystem::now();
    if(m_type != OTHER) m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    m_text->appendStyledText(nick+": ", style);
    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, FALSE);
    appendLinkText(msg, 0);
    if(logLine) this->logLine("<"+nick+"> "+msg, time);
}

void IrcTabItem::appendMyMsg(FXString msg, FXTime time)
{
    if(!time) time = FXSystem::now();
    m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    m_text->appendStyledText(getNickName()+": ", preferences.m_coloredNick?10:5);
    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, FALSE);
    appendLinkText(msg, 10);
    this->logLine("\01410\014<"+getNickName()+"> "+msg, time);
}

/* if highlight==TRUE, highlight tab
 * disableStrip is for dxirc.Print
*/
void IrcTabItem::appendStyledText(FXString text, FXint style, FXbool highlight, FXbool disableStrip, FXbool logLine)
{
    if(style) appendIrcStyledText(text, style, 0, disableStrip, logLine);
    else appendIrcText(text, 0, disableStrip, logLine);
    if(highlight && preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
    {
        if(m_type != OTHER && highlightNeeded(text))
        {
            this->setTextColor(preferences.m_highlightColor);
            if(m_type == CHANNEL) this->setIcon(ICO_CHANNELNEWMSG);
        }
        else this->setTextColor(preferences.m_unreadColor);
        if(m_type == QUERY) this->setIcon(ICO_QUERYNEWMSG);
    }
}

void IrcTabItem::appendIrcStyledText(FXString styled, FXint stylenum, FXTime time, FXbool disableStrip, FXbool logLine)
{
    if(!time) time = FXSystem::now();
    if(m_type != OTHER) m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    if(preferences.m_stripColors && !disableStrip) styled = dxutils::stripColors(styled, TRUE);
    appendLinkText(styled, stylenum);
    if(logLine)
    {
        if(stylenum==8) //highlight msg
        {
            this->logLine("\0148\014<"+styled.before(':')+">"+styled.after(':'), time);
        }
        else this->logLine(FXStringFormat("\014%d\014%s", stylenum, styled.text()), time);
    }
}

static FXbool isBadchar(FXchar c)
{
    switch(c) {
        case ' ':
        case ',':
        case '\0':
        case '\02':
        case '\03':
        case '\017':
        case '\021':
        case '\026':
        case '\035':
        case '\037':
        case '\n':
        case '\r':
        case '<':
        case '>':
        case '"':
        case '\'':
            return TRUE;
        default:
            return FALSE;
    }
}

// checks is char nick/word delimiter
static FXbool isDelimiter(FXchar c)
{
    switch(c) {
        case ' ':
        case '.':
        case ',':
        case '/':
        case '\\':
        case '`':
        case '\'':
        case '!':
        case '(':
        case ')':
        case '{':
        case '}':
        case '|':
        case '[':
        case ']':
        case '\"':
        case ':':
        case ';':
        case '<':
        case '>':
        case '?':
            return TRUE;
        default:
            return FALSE;
    }
}

void IrcTabItem::appendLinkText(const FXString &txt, FXint stylenum)
{
    FXint i = 0;
    FXint linkLength = 0;
    FXbool bold = FALSE;
    FXbool under = FALSE;
    FXint lastStyle = stylenum;
    FXColor foreColor = stylenum && stylenum<=m_textStyleList.no() ? m_textStyleList[stylenum-1].normalForeColor : preferences.m_colors.text;
    FXColor backColor = stylenum && stylenum<=m_textStyleList.no() ? m_textStyleList[stylenum-1].normalBackColor : preferences.m_colors.back;
    FXString normalText = "";
    FXint length = txt.length();
    while(i<length)
    {
        if(txt[i]=='h' && !comparecase(txt.mid(i,7),"http://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>7 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='h' && !comparecase(txt.mid(i,8),"https://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>8 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='f' && !comparecase(txt.mid(i,6),"ftp://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>6 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='w' && !comparecase(txt.mid(i,4),"www."))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>4 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else
        {
            if(txt[i] == '\002') //bold
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                bold = !bold;
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\026') //reverse
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXColor tempColor = foreColor;
                foreColor = backColor;
                backColor = tempColor;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\037') //underline
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                under = !under;
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\021') //fixed
            {
                dxutils::debugLine("Poslan fixed styl");
            }
            else if(txt[i] == '\035') //italic
            {
                dxutils::debugLine("Poslan italic styl");
            }
            else if(txt[i] == '\003') //color
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style=0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXbool isHexColor = FALSE;
                FXint colorLength = 0;
                foreColor = preferences.m_colors.text;
                backColor = preferences.m_colors.back;
                if(i+1<length)
                {
                    if(txt[i+1] == '#') isHexColor = TRUE;
                }
                if(isHexColor)
                {
                    if(FXRex("^\\h\\h\\h\\h\\h\\h+$").match(txt.mid(i+2,6)))
                    {
                        foreColor = FXRGB(FXIntVal(txt.mid(i+2,2),16),FXIntVal(txt.mid(i+4,2),16),FXIntVal(txt.mid(i+6,2),16));
                        colorLength +=7;
                    }
                    if(i+8 < length && txt[i+8] == ',' && FXRex("^\\h\\h\\h\\h\\h\\h+$").match(txt.mid(i+10,6)))
                    {
                        backColor = FXRGB(FXIntVal(txt.mid(i+10,2),16),FXIntVal(txt.mid(i+12,2),16),FXIntVal(txt.mid(i+14,2),16));
                        colorLength +=8;
                    }
                }
                else
                {
                    if(i+2<length)
                    {
                        FXint code = -1;
                        if(isdigit(txt[i+1]))
                        {
                            if(isdigit(txt[i+2]))
                            {
                                code = (txt[i+1]-48)*10+txt[i+2]-48;
                                colorLength +=2;
                            }
                            else
                            {
                                code = txt[i+1]-48;
                                colorLength ++;
                            }
                        }
                        if(code!=-1)
                            foreColor = dxutils::getIrcColor(code%16);
                    }
                    if(i+colorLength+1 < length && txt[i+colorLength+1] == ',')
                    {
                        FXint code = -1;
                        if(isdigit(txt[i+colorLength+2]))
                        {
                            if(isdigit(txt[i+colorLength+3]))
                            {
                                code = (txt[i+colorLength+2]-48)*10+txt[i+colorLength+3]-48;
                                colorLength +=3;
                            }
                            else
                            {
                                code = txt[i+colorLength+2]-48;
                                colorLength +=2;
                            }
                        }
                        if(code!=-1)
                            backColor = dxutils::getIrcColor(code%16);
                    }
                }
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                };
                i +=colorLength;
            }
            else if(txt[i] == '\017') //reset
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                bold = FALSE;
                under = FALSE;
                foreColor = preferences.m_colors.text;
                backColor = preferences.m_colors.back;
                lastStyle = stylenum;
            }
            else
            {
                normalText.append(txt[i]);
            }
        }
        i++;
    }
    if(!normalText.empty())
    {
        m_text->appendStyledText(normalText, lastStyle);
    }
    m_text->appendText("\n");
}

void IrcTabItem::startLogging()
{
    if(m_logstream && FXStat::exists(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText()+PATHSEPSTRING+FXSystem::time("%Y-%m-%d", FXSystem::now())))
        return;
    if(preferences.m_logging && m_type != SERVER)
    {
        if(!FXStat::exists(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName())) FXDir::create(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName());
        if(!FXStat::exists(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText())) FXDir::create(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText());
        m_logstream = new std::ofstream(FXString(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText()+PATHSEPSTRING+FXSystem::time("%Y-%m-%d", FXSystem::now())).text(), std::ios::out|std::ios::app);
    }
}

void IrcTabItem::stopLogging()
{
    if(m_logstream)
    {
        m_logstream->close();
        delete m_logstream;
        m_logstream = NULL;
    }
}

void IrcTabItem::logLine(const FXString &line, const FXTime &time)
{
    if(preferences.m_logging && m_type != SERVER)
    {
        this->startLogging();
        *m_logstream << FXSystem::time("[%H:%M:%S] ", time).text() << line.text() << std::endl;
    }
}

FXbool IrcTabItem::isChannel(const FXString &text)
{
    if(text.length()) return m_engine->getChanTypes().contains(text[0]);
    return FALSE;
}

long IrcTabItem::onCommandline(FXObject *, FXSelector, void *)
{
    FXString commandtext = m_commandline->getText();
    if(commandtext.empty())
        return 1;
    m_commandsHistory.append(commandtext);
    if (m_commandsHistory.no() > m_historyMax)
        m_commandsHistory.erase(0);
    m_currentPosition = m_commandsHistory.no()-1;
    m_commandline->setText("");
    if(comparecase(commandtext.left(4),"/say") != 0)
        commandtext.substitute("^B", "\002").substitute("^C", "\003").substitute("^O", "\017").substitute("^V", "\026").substitute("^_", "\037");
    for(FXint i=0; i<=commandtext.contains('\n'); i++)
    {
        FXString text = commandtext.section('\n', i).before('\r');
        if(comparecase(text.after('/').before(' '), "quit") == 0 ||
                comparecase(text.after('/').before(' '), "lua") == 0)
        {
            processLine(text);
            return 1;
        }
#ifdef HAVE_LUA
        if(text[0] != '/') m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_OWNMSG), &text);
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_COMMAND), &text);
        if(text[0] == '/' && !m_scriptHasAll) processLine(text);
        else if(!m_scriptHasOwnMsg && !m_scriptHasAll) processLine(text);
#else
        processLine(text);
#endif
    }
    return 1;
}

FXbool IrcTabItem::processLine(const FXString& commandtext)
{
    FXString command = (commandtext[0] == '/' ? commandtext.before(' ') : "");
    if(!dxUtils.getAlias(command).empty())
    {
        FXString acommand = dxUtils.getAlias(command);
        if(acommand.contains("%s"))
            acommand.substitute("%s", commandtext.after(' '));
        else
            acommand += commandtext.after(' ');
        FXint num = acommand.contains("&&");
        if(num)
        {
            FXbool result = FALSE;
            for(FXint i=0; i<=acommand.contains('&'); i++)
            {
                if(!acommand.section('&',i).trim().empty()) result = processCommand(acommand.section('&',i).trim());
            }
            return result;
        }
        else
        {
            return processCommand(acommand);
        }
    }
    return processCommand(commandtext);
}

FXbool IrcTabItem::processCommand(const FXString& commandtext)
{
    FXString command = (commandtext[0] == '/' ? commandtext.after('/').before(' ').lower() : "");
    if(m_type == OTHER)
    {
        if(dxUtils.isScriptCommand(command))
        {
            return commandScript(commandtext);
        }
        if(command == "commands")
        {
            return commandCommands();
        }
        if(command == "egg")
        {
            return commandEgg();
        }
        if(command == "help")
        {
            return commandHelp(commandtext);
        }
        if(command == "tetris")
        {
            return commandTetris();
        }
        return TRUE;
    }
    if(commandtext[0] == '/')
    {
        if(dxUtils.isScriptCommand(command))
        {
            return commandScript(commandtext);
        }
        if(command == "admin")
        {
            return commandAdmin(commandtext);
        }
        if(command == "away")
        {
            return commandAway(commandtext);
        }
        if(command == "banlist")
        {
            return commandBanlist(commandtext);
        }
        if(command == "boats")
        {
            return commandBoats(commandtext);
        }
        if(command == "connect")
        {
            return commandConnect(commandtext);
        }
        if(command == "commands")
        {
            return commandCommands();
        }
        if(command == "ctcp")
        {
            return commandCtcp(commandtext);
        }
        if(command == "cycle")
        {
            return commandCycle(commandtext);
        }
        if(command == "dcc")
        {
            return commandDcc(commandtext);
        }
        if(command == "deop")
        {
            return commandDeop(commandtext);
        }
        if(command == "devoice")
        {
            return commandDevoice(commandtext);
        }
        if(command == "disconnect")
        {
            return commandDisconnect(commandtext);
        }
        if(command == "dxirc")
        {
            return commandDxirc();
        }
        if(command == "egg")
        {
            return commandEgg();
        }
        if(command == "exec")
        {
            return commandExec(commandtext);
        }
        if(command == "help")
        {
            return commandHelp(commandtext);
        }
        if(command == "ignore")
        {
            return commandIgnore(commandtext);
        }
        if(command == "invite")
        {
            return commandInvite(commandtext);
        }
        if(command == "join")
        {
            return commandJoin(commandtext);
        }
        if(command == "kick")
        {
            return commandKick(commandtext);
        }
        if(command == "kill")
        {
            return commandKill(commandtext);
        }
        if(command == "list")
        {
            return commandList(commandtext);
        }
        if(command == "log")
        {
            return commandLog(commandtext);
        }
        if(command == "lua")
        {
            return commandLua(commandtext);
        }
        if(command == "me")
        {
            return commandMe(commandtext);
        }
        if(command == "mode")
        {
            return commandMode(commandtext);
        }
        if(command == "msg")
        {
            return commandMsg(commandtext);
        }
        if(command == "names")
        {
            return commandNames(commandtext);
        }
        if(command == "nick")
        {
            return commandNick(commandtext);
        }
        if(command == "notice")
        {
            return commandNotice(commandtext);
        }
        if(command == "op")
        {
            return commandOp(commandtext);
        }
        if(command == "oper")
        {
            return commandOper(commandtext);
        }
        if(command == "part")
        {
            return commandPart(commandtext);
        }
        if(command == "query")
        {
            return commandQuery(commandtext);
        }
        if(command == "quit")
        {
            return commandQuit();
        }
        if(command == "quote")
        {
            return commandQuote(commandtext);
        }
        if(command == "say")
        {
            return commandSay(commandtext);
        }
        if(command == "stats")
        {
            return commandStats(commandtext);
        }
        if(command == "tetris")
        {
            return commandTetris();
        }
        if(command == "time")
        {
            return commandTime();
        }
        if(command == "topic")
        {
            return commandTopic(commandtext);
        }
        if(command == "voice")
        {
            return commandVoice(commandtext);
        }
        if(command == "wallops")
        {
            return commandWallops(commandtext);
        }
        if(command == "who")
        {
            return commandWho(commandtext);
        }
        if(command == "whoami")
        {
            return commandWhoami();
        }
        if(command == "whois")
        {
            return commandWhois(commandtext);
        }
        if(command == "whowas")
        {
            return commandWhowas(commandtext);
        }
        appendIrcStyledText(FXStringFormat(_("Unknown command '%s', type /commands for available commands"), command.text()), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
    else
    {
        if(command.empty() && m_type != SERVER && !commandtext.empty() && m_engine->getConnected())
        {
            if(commandtext.length() > m_maxLen-10-getText().length())
            {
                dxStringArray messages = cutText(commandtext, m_maxLen-10-getText().length());
                FXbool result = TRUE;
                for(FXint i=0; i<messages.no(); i++)
                {
                    appendMyMsg(messages[i], FXSystem::now());
                    IrcEvent ev;
                    ev.eventType = IRC_PRIVMSG;
                    ev.param1 = getNickName();
                    ev.param2 = getText();
                    ev.param3 = messages[i];
                    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                    result = m_engine->sendMsg(getText(), messages[i]) &result;
                }
                return result;
            }
            else
            {
                appendMyMsg(commandtext, FXSystem::now());
                IrcEvent ev;
                ev.eventType = IRC_PRIVMSG;
                ev.param1 = getNickName();
                ev.param2 = getText();
                ev.param3 = commandtext;
                m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                return m_engine->sendMsg(getText(), commandtext);
            }
        }
        if(!m_engine->getConnected())
        {
            appendIrcStyledText(_("You aren't connected"), 4, FXSystem::now(), FALSE, FALSE);
            m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_CDIALOG), NULL);
            return TRUE;
        }
        return FALSE;
    }
    return FALSE;
}

FXbool IrcTabItem::showHelp(FXString command)
{
    if(dxUtils.isScriptCommand(command))
    {
        appendIrcStyledText(dxUtils.getHelpText(command), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "admin")
    {
        appendIrcStyledText(_("ADMIN [server], finds information about administrator for current server or [server]."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "away")
    {
        appendIrcStyledText(_("AWAY [message], sets away status."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "banlist")
    {
        appendIrcStyledText(_("BANLIST <channel>, shows banlist for channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "boats")
    {
        appendIrcStyledText(_("BOATS <nick>, invites someone to a boats game."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "commands")
    {
        appendIrcStyledText(_("COMMANDS, shows available commands"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "connect")
    {
        appendIrcStyledText(_("CONNECT <server> [port] [nick] [password] [realname] [channels], connects for given server."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "ctcp")
    {
        appendIrcStyledText(_("CTCP <nick> <message>, sends a CTCP message to a user."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "cycle")
    {
        appendIrcStyledText(_("CYCLE <channel> [message], leaves and join channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "dcc")
    {
        appendIrcStyledText(_("DCC chat <nick>, starts DCC chat."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("DCC send <nick> <filename>, sends file over DCC."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("DCC psend <nick> <filename>, sends file passive over DCC."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("More information about passive DCC on http://en.wikipedia.org/wiki/Direct_Client-to-Client#Passive_DCC"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "deop")
    {
        appendIrcStyledText(_("DEOP <channel> <nicks>, removes operator status from one or more nicks."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "devoice")
    {
        appendIrcStyledText(_("DEVOICE <channel> <nicks>, removes voice from one or more nicks."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "disconnect")
    {
        appendIrcStyledText(_("DISCONNECT [reason], leaves server."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#ifndef WIN32
    if(command == "exec")
    {
        appendIrcStyledText(_("EXEC [-o|-c] <command>, executes command, -o sends output to channel/query, -c closes running command."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#endif
    if(command == "help")
    {
        appendIrcStyledText(_("HELP <command>, shows help for command."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "ignore")
    {
        appendIrcStyledText(_("IGNORE list, shows list ignored commands and users."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("IGNORE addcmd <command>, adds command to ignored commands."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("IGNORE rmcmd <command>, removes command from ignored commands."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("IGNORE addusr <user> [channel] [server], adds user to ignored users."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("IGNORE rmusr <user>, removes user from ignored users."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "invite")
    {
        appendIrcStyledText(_("INVITE <nick> <channel>, invites someone to a channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "join")
    {
        appendIrcStyledText(_("JOIN <channel>, joins a channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "kick")
    {
        appendIrcStyledText(_("KICK <channel> <nick>, kicks a user from a channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "kill")
    {
        appendIrcStyledText(_("KILL <user> [reason], kills a user from the network."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "list")
    {
        appendIrcStyledText(_("LIST [channel], lists channels and their topics."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "log")
    {
        appendIrcStyledText(_("LOG list, shows list of log files for current tab"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LOG show [date], shows last log file for current tab, if date is empty"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("else file for given date"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("date example: 2011-01-31"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#ifdef HAVE_LUA
    if(command == "lua")
    {
        appendIrcStyledText(_("LUA help, shows help for lua scripting."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LUA load <path>, loads script."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Example: /lua load /home/dvx/test.lua"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LUA unload <name>, unloads script."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Example: /lua unload test"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LUA list, shows list of loaded scripts"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#else
    if(command == "lua")
    {
        appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#endif
    if(command == "me")
    {
        appendIrcStyledText(_("ME <to> <message>, sends the action."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "mode")
    {
        appendIrcStyledText(_("MODE <channel> <modes>, sets modes for a channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "msg")
    {
        appendIrcStyledText(_("MSG <nick/channel> <message>, sends a normal message."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "names")
    {
        appendIrcStyledText(_("NAMES <channel>, for nicks on a channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "nick")
    {
        appendIrcStyledText(_("NICK <nick>, changes nick."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "notice")
    {
        appendIrcStyledText(_("NOTICE <nick/channel> <message>, sends a notice."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "op")
    {
        appendIrcStyledText(_("OP <channel> <nicks>, gives operator status for one or more nicks."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "oper")
    {
        appendIrcStyledText(_("OPER <login> <password>, oper up."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "part")
    {
        appendIrcStyledText(_("PART <channel> [reason], leaves channel."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "query")
    {
        appendIrcStyledText(_("QUERY <nick>, opens query with nick."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "quit")
    {
        appendIrcStyledText(_("QUIT, closes application."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "quote")
    {
        appendIrcStyledText(_("QUOTE [text], sends text to server."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "say")
    {
        appendIrcStyledText(_("SAY [text], sends text to current tab."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "stats")
    {
        appendIrcStyledText(_("STATS <type>, shows some irc server usage statistics. Available types vary slightly per server; some common ones are:"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("c - shows C and N lines for a given server.  These are the names of the servers that are allowed to connect."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("h - shows H and L lines for a given server (Hubs and Leaves)."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("k - show K lines for a server.  This shows who is not allowed to connect and possibly at what time they are not allowed to connect."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("i - shows I lines. This is who CAN connect to a server."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("l - shows information about amount of information passed to servers and users."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("m - shows a count for the number of times the various commands have been used since the server was booted."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("o - shows the list of authorized operators on the server."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("p - shows online operators and their idle times."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("u - shows the uptime for a server."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("y - shows Y lines, which lists the various connection classes for a given server."), 3, FXSystem::now(), FALSE, FALSE);

        return TRUE;
    }
    if(command == "tetris")
    {
        appendIrcStyledText(_("TETRIS, start small easteregg."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Keys for playing:"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("n .. new game"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("p .. pause game"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("i .. rotate piece"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("l .. move piece right"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("k .. drop piece"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("j .. move piece left"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "time")
    {
        appendIrcStyledText(_("TIME, displays the time of day, local to server."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "topic")
    {
        appendIrcStyledText(_("TOPIC [topic], sets or shows topic."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "voice")
    {
        appendIrcStyledText(_("VOICE <channel> <nicks>, gives voice for one or more nicks."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "wallops")
    {
        appendIrcStyledText(_("WALLOPS <message>, sends wallop message."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "who")
    {
        appendIrcStyledText(_("WHO <mask> [o], searchs for mask on network, if o is supplied, only search for opers."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "whoami")
    {
        appendIrcStyledText(_("WHOAMI, whois about you."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "whois")
    {
        appendIrcStyledText(_("WHOIS <nick>, whois nick."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "whowas")
    {
        appendIrcStyledText(_("WHOWAS <nick>, whowas nick."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(!dxUtils.getAlias(command[0] == '/' ? command:"/"+command).empty())
    {
        appendIrcStyledText(FXStringFormat("%s: %s", command.upper().text(), dxUtils.getAlias(command[0] == '/' ? command:"/"+command).text()), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command.empty()) appendIrcStyledText(_("Command is empty, type /commands for available commands"), 4, FXSystem::now(), FALSE, FALSE);
    else appendIrcStyledText(FXStringFormat(_("Unknown command '%s', type /commands for available commands"), command.text()), 4, FXSystem::now(), FALSE, FALSE);
    return FALSE;
}

long IrcTabItem::onKeyPress(FXObject *, FXSelector, void *ptr)
{
    if(m_commandline->hasFocus())
    {
        FXString line = m_commandline->getText();
        FXString key = dxutils::getKeybinding((FXEvent*)ptr);
        if(comparecase(key, "Tab")==0) //tab completion
        {
            if(comparecase(line.before(' '), "/help") == 0 && !line.after(' ').empty())
            {
                completeHelpCommand();
                return 1;
            }
            if(line[0] == '/' && line.after(' ').empty())
            {
                for(FXint i = 0; i < dxUtils.commandsNo(); i++)
                {
                    if(comparecase(line.after('/').before(' '), dxUtils.commandsAt(i)) == 0)
                    {
                        if((i+1) < dxUtils.commandsNo()) m_commandline->setText("/"+dxUtils.commandsAt(++i)+" ");
                        else m_commandline->setText("/"+dxUtils.commandsAt(0)+" ");
                        break;
                    }
                    else if(comparecase(line.after('/'), dxUtils.commandsAt(i).left(line.after('/').length())) == 0)
                    {
                        m_commandline->setText("/"+dxUtils.commandsAt(i)+" ");
                        break;
                    }
                }
                return 1;
            }
            if(line[0] != '/' && line.after(' ').empty() && m_type == CHANNEL)
            {
                if(line.empty())
                {
                    m_commandline->setText(getNick(0)+preferences.m_nickCompletionChar+" ");
                    return 1;
                }
                for(FXint j = 0; j < m_users->getNumItems() ; j++)
                {
                    if(comparecase(line, getNick(j).left(line.length())) == 0)
                    {
                        m_commandline->setText(getNick(j)+preferences.m_nickCompletionChar+" ");
                        break;
                    }
                    else if(comparecase(line.section(preferences.m_nickCompletionChar, 0, 1), getNick(j)) == 0)
                    {
                        if((j+1) < m_users->getNumItems()) m_commandline->setText(getNick(++j)+preferences.m_nickCompletionChar+" ");
                        else m_commandline->setText(getNick(0)+preferences.m_nickCompletionChar+" ");
                        break;
                    }
                }
                return 1;
            }
            if(line.find(' ') != -1 && m_type == CHANNEL)
            {
                FXint curpos;
                line[m_commandline->getCursorPos()] == ' ' ? curpos = m_commandline->getCursorPos()-1 : curpos = m_commandline->getCursorPos();
                FXint pos = line.rfind(' ', curpos)+1;
                FXint n = line.find(' ', curpos)>0 ? line.find(' ', curpos)-pos : line.length()-pos;
                FXString toCompletion = line.mid(pos, n);
                for(FXint j = 0; j < m_users->getNumItems(); j++)
                {
                    if(comparecase(toCompletion, getNick(j)) == 0)
                    {
                        if((j+1) < m_users->getNumItems())
                        {
                            m_commandline->setText(line.replace(pos, n, getNick(j+1)));
                            m_commandline->setCursorPos(pos+getNick(j+1).length());
                        }
                        else
                        {
                            m_commandline->setText(line.replace(pos, n, getNick(0)));
                            m_commandline->setCursorPos(pos+getNick(0).length());
                        }
                        break;
                    }
                    else if(comparecase(toCompletion, getNick(j).left(toCompletion.length())) == 0)
                    {
                        m_commandline->setText(line.replace(pos, n, getNick(j)));
                        m_commandline->setCursorPos(pos+getNick(j).length());
                        break;
                    }
                }
                return 1;
            }
            return 1;
        }
        else if(comparecase(key, "Up")==0) //append history command backward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition > 0 && !line.empty())
                    --m_currentPosition;
                m_commandline->setText(m_commandsHistory[m_currentPosition]);
            }
            return 1;
        }
        else if(comparecase(key, "Down")==0) //append history command forward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition < m_commandsHistory.no()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_commandsHistory[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
            return 1;
        }
        else if(comparecase(key, preferences.getBinding("colorControlChar"))==0) //color control char
        {
            FXint pos = m_commandline->getCursorPos();
            m_commandline->setText(line.insert(pos, "^C")); //color
            m_commandline->setCursorPos(pos+2);
            return 1;
        }
        else if(comparecase(key, preferences.getBinding("boldControlChar"))==0) //bold control char
        {
            FXint pos = m_commandline->getCursorPos();
            m_commandline->setText(line.insert(pos, "^B")); //bold
            m_commandline->setCursorPos(pos+2);
            return 1;
        }
        else if(comparecase(key, preferences.getBinding("underlineControlChar"))==0) //underline control char
        {
            FXint pos = m_commandline->getCursorPos();
            m_commandline->setText(line.insert(pos, "^_")); //underline
            m_commandline->setCursorPos(pos+2);
            return 1;
        }
        else if(comparecase(key, preferences.getBinding("reverseControlChar"))==0) //reverse control char
        {
            FXint pos = m_commandline->getCursorPos();
            m_commandline->setText(line.insert(pos, "^V")); //reverse
            m_commandline->setCursorPos(pos+2);
            return 1;
        }
        else if(comparecase(key, preferences.getBinding("resetControlChar"))==0) //reset control char
        {
            FXint pos = m_commandline->getCursorPos();
            m_commandline->setText(line.insert(m_commandline->getCursorPos(), "^O")); //reset
            m_commandline->setCursorPos(pos+2);
            return 1;
        }
    }
    return 0;
}

//Check is this tab right for server message (e.g. not for IRC_SERVERREPLY)
FXbool IrcTabItem::isRightForServerMsg()
{
    //Check if current tab is in server targets
    if(m_engine->findTarget(m_parent->childAtIndex(m_parent->getCurrent()*2)))
    {
        return m_parent->indexOfChild(this) == m_parent->getCurrent()*2;
    }
    else
    {
        FXint indexOfThis = m_parent->indexOfChild(this);
        for(FXint i = 0; i<m_parent->numChildren(); i+=2)
        {
            if(m_engine->findTarget(m_parent->childAtIndex(i)))
            {
                if(i == indexOfThis) return TRUE;
                else return FALSE;
            }
        }
    }
    return FALSE;
}

FXbool IrcTabItem::isCommandIgnored(const FXString &command)
{
    if(preferences.m_ignoreCommandsList.contains(command)) return TRUE;
    return FALSE;
}

void IrcTabItem::addUser(const FXString& user, UserMode mode)
{
    if(mode == ADMIN && m_users->findItem(user) == -1)
    {
        NickListItem *item = new NickListItem(user, m_users, mode, m_useNickIcon?ICO_IRCADMIN:NULL);
        m_users->appendItem(item);
        m_numberUsers++;
        m_users->sortItems();
        return;
    }
    if(mode == OWNER && m_users->findItem(user) == -1)
    {
        NickListItem *item = new NickListItem(user, m_users, mode, m_useNickIcon?ICO_IRCOWNER:NULL);
        m_users->appendItem(item);
        m_numberUsers++;
        m_users->sortItems();
        return;
    }
    if(mode == OP && m_users->findItem(user) == -1)
    {
        NickListItem *item = new NickListItem(user, m_users, mode, m_useNickIcon?ICO_IRCOP:NULL);
        m_users->appendItem(item);
        m_numberUsers++;
        m_users->sortItems();
        return;
    }
    if(mode == VOICE && m_users->findItem(user) == -1)
    {
        NickListItem *item = new NickListItem(user, m_users, mode, m_useNickIcon?ICO_IRCVOICE:NULL);
        m_users->appendItem(item);
        m_numberUsers++;
        m_users->sortItems();
        return;
    }
    if(mode == HALFOP && m_users->findItem(user) == -1)
    {
        NickListItem *item = new NickListItem(user, m_users, mode, m_useNickIcon?ICO_IRCHALFOP:NULL);
        m_users->appendItem(item);
        m_numberUsers++;
        m_users->sortItems();
        return;
    }
    if(mode == NONE && m_users->findItem(user) == -1)
    {
        NickListItem *item = new NickListItem(user, m_users, mode, m_useNickIcon?ICO_IRCNORMAL:NULL);
        m_users->appendItem(item);
        m_numberUsers++;
        m_users->sortItems();
        return;
    }
}

void IrcTabItem::removeUser(const FXString& user)
{
    if(m_users->findItem(user) != -1)
    {
        m_users->removeItem(m_users->findItem(user));
    }
    m_numberUsers--;
    m_users->sortItems();
}

void IrcTabItem::changeNickUser(const FXString& nick, const FXString& newnick)
{
    FXint i = m_users->findItem(nick);
    if(i != -1)
    {
        m_users->recalc();
        m_users->getItem(i)->setText(newnick);
        m_users->sortItems();
    }
}

void IrcTabItem::setUserMode(const FXString& nick, UserMode mode)
{
    FXint i = m_users->findItem(nick);
    if(i != -1)
    {
        ((NickListItem*)m_users->getItem(i))->setUserMode(mode);
    }
}

UserMode IrcTabItem::getUserMode(const FXchar mode)
{
    if(mode == m_engine->getAdminPrefix())
        return ADMIN;
    if(mode == m_engine->getOwnerPrefix())
        return OWNER;
    if(mode == m_engine->getOpPrefix())
        return OP;
    if(mode == m_engine->getHalfopPrefix())
        return HALFOP;
    if(mode == m_engine->getVoicePrefix())
        return VOICE;
    return NONE;
}

//Clear theme icons, usefull for nick icon theme change
void IrcTabItem::clearNickIcons()
{
    for(FXint i=0; i<m_users->getNumItems(); i++)
    {
        ((NickListItem*)m_users->getItem(i))->clearIcon();
    }
}

//Apply new theme icon, usefull for nick icon theme change
void IrcTabItem::recreateNickIcons(FXbool useNickIcon)
{
    m_useNickIcon = useNickIcon;
    if(m_useNickIcon)
    {
        for(FXint i=0; i<m_users->getNumItems(); i++)
        {
            ((NickListItem*)m_users->getItem(i))->restoreIcon();
        }
    }
}

//return names
dxStringArray IrcTabItem::getNicks()
{
    dxStringArray names;
    for(FXint i=0; i<m_users->getNumItems(); i++)
        names.append(m_users->getItemText(i));
    return names;
}

FXbool IrcTabItem::hasNick(const FXString& nick)
{
    return m_users->findItem(nick) != -1;
}

long IrcTabItem::onIrcEvent(FXObject *, FXSelector, void *data)
{
    IrcEvent ev = *(IrcEvent *)data;
    if(m_loadLog)
    {
        m_storedEvents.append(ev);
        return 1;
    }
    parseIrcEvent(ev);
    return 1;
}

void IrcTabItem::parseIrcEvent(IrcEvent &ev)
{
    if(ev.eventType == IRC_PRIVMSG)
    {
        onIrcPrivmsg(ev);
    }
    else if(ev.eventType == IRC_ACTION)
    {
        onIrcAction(ev);
    }
    else if(ev.eventType == IRC_CTCPREPLY)
    {
        onIrcCtpcReply(ev);
    }
    else if(ev.eventType == IRC_CTCPREQUEST)
    {
        onIrcCtcpRequest(ev);
    }
    else if(ev.eventType == IRC_JOIN)
    {
        onIrcJoin(ev);
    }
    else if(ev.eventType == IRC_QUIT)
    {
        onIrcQuit(ev);
    }
    else if(ev.eventType == IRC_PART)
    {
        onIrcPart(ev);
    }
    else if(ev.eventType == IRC_CHNOTICE)
    {
        onIrcChnotice(ev);
    }
    else if(ev.eventType == IRC_NOTICE)
    {
        onIrcNotice(ev);
    }
    else if(ev.eventType == IRC_NICK)
    {
        onIrcNick(ev);
    }
    else if(ev.eventType == IRC_TOPIC)
    {
        onIrcTopic(ev);
    }
    else if(ev.eventType == IRC_KICK)
    {
        onIrcKick(ev);
    }
    else if(ev.eventType == IRC_MODE)
    {
        onIrcMode(ev);
    }
    else if(ev.eventType == IRC_UMODE)
    {
        onIrcUmode(ev);
    }
    else if(ev.eventType == IRC_CHMODE)
    {
        onIrcChmode(ev);
    }
    else if(ev.eventType == IRC_SERVERREPLY)
    {
        onIrcServerReply(ev);
    }
    else if(ev.eventType == IRC_CONNECT)
    {
        onIrcConnect(ev);
    }
    else if(ev.eventType == IRC_ERROR)
    {
        onIrcError(ev);
    }
    else if(ev.eventType == IRC_SERVERERROR)
    {
        onIrcServerError(ev);
    }
    else if(ev.eventType == IRC_DISCONNECT)
    {
        onIrcDisconnect(ev);
    }
    else if(ev.eventType == IRC_RECONNECT)
    {
        onIrcReconnect(ev);
    }
    else if(ev.eventType == IRC_RECONNECTALL)
    {
        onIrcReconnectAll(ev);
    }
    else if(ev.eventType == IRC_UNKNOWN)
    {
        onIrcUnknown(ev);
    }
    else if(ev.eventType == IRC_301)
    {
        onIrc301(ev);
    }
    else if(ev.eventType == IRC_305)
    {
        onIrc305(ev);
    }
    else if(ev.eventType == IRC_306)
    {
        onIrc306(ev);
    }
    else if(ev.eventType == IRC_331 || ev.eventType == IRC_332 || ev.eventType == IRC_333)
    {
        onIrc331332333(ev);
    }
    else if(ev.eventType == IRC_353)
    {
        onIrc353(ev);
    }
    else if(ev.eventType == IRC_366)
    {
        onIrc366(ev);
    }
    else if(ev.eventType == IRC_372)
    {
        onIrc372(ev);
    }
    else if(ev.eventType == IRC_AWAY)
    {
        onIrcAway(ev);
    }
    else if(ev.eventType == IRC_ENDMOTD)
    {
        onIrcEndMotd();
    }
}

//handle IrcEvent IRC_PRIVMSG
void IrcTabItem::onIrcPrivmsg(IrcEvent &ev)
{
    if((comparecase(ev.param2, getText()) == 0 && m_type == CHANNEL) || (ev.param1 == getText() && m_type == QUERY && ev.param2 == getNickName()))
    {
        FXbool needHighlight = FALSE;
        if(ev.param3.contains(getNickName()))
            needHighlight = highlightNeeded(ev.param3);
        if(needHighlight) appendIrcStyledText(ev.param1+": "+ev.param3, 8, ev.time);
        else appendIrcNickText(ev.param1, ev.param3, getNickStyle(ev.param1), ev.time);
        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
        {
            if(needHighlight)
            {
                this->setTextColor(preferences.m_highlightColor);
                if(m_type == CHANNEL) this->setIcon(ICO_CHANNELNEWMSG);
            }
            else this->setTextColor(preferences.m_unreadColor);
            if(m_type == QUERY) this->setIcon(ICO_QUERYNEWMSG);
        }
        if((m_type == CHANNEL && needHighlight) || m_type == QUERY)
            m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_NEWMSG), NULL);
    }
}

//handle IrcEvent IRC_ACTION
void IrcTabItem::onIrcAction(IrcEvent &ev)
{
    if((comparecase(ev.param2, getText()) == 0 && m_type == CHANNEL) || (ev.param1 == getText() && m_type == QUERY && ev.param2 == getNickName()))
    {
        if(!isCommandIgnored("me"))
        {
            FXbool needHighlight = FALSE;
            if(ev.param3.contains(getNickName()))
                needHighlight = highlightNeeded(ev.param3);
            appendIrcStyledText(ev.param1+" "+ev.param3, 2, ev.time);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
            {
                if(needHighlight)
                {
                    this->setTextColor(preferences.m_highlightColor);
                    if(m_type == CHANNEL) this->setIcon(ICO_CHANNELNEWMSG);
                }
                else this->setTextColor(preferences.m_unreadColor);
                if(m_type == QUERY) this->setIcon(ICO_QUERYNEWMSG);
            }
            if((m_type == CHANNEL && needHighlight) || m_type == QUERY)
                m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_NEWMSG), NULL);
        }
    }
}

//handle IrcEvent IRC_CTCPREPLY
void IrcTabItem::onIrcCtpcReply(IrcEvent &ev)
{
    if(isRightForServerMsg())
    {
        if(!isCommandIgnored("ctcp"))
        {
            appendIrcStyledText(FXStringFormat(_("CTCP %s reply from %s: %s"), dxutils::getParam(ev.param2, 1, FALSE).text(), ev.param1.text(), dxutils::getParam(ev.param2, 2, TRUE).text()), 2, ev.time, FALSE, FALSE);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }
    }
}

//handle IrcEvent IRC_CTCPREQUEST
void IrcTabItem::onIrcCtcpRequest(IrcEvent &ev)
{
    if(isRightForServerMsg())
    {
        if(!isCommandIgnored("ctcp"))
        {
            appendIrcStyledText(FXStringFormat(_("CTCP %s request from %s"), ev.param2.text(), ev.param1.text()), 2, ev.time, FALSE, FALSE);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }
    }
}

//handle IrcEvent IRC_JOIN
void IrcTabItem::onIrcJoin(IrcEvent &ev)
{
    if(comparecase(ev.param2, getText()) == 0 && ev.param1 != getNickName())
    {
        if(!isCommandIgnored("join") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s has joined to %s"), ev.param1.text(), ev.param2.text()), 1, ev.time);
        addUser(ev.param1, NONE);
    }
}

//handle IrcEvent IRC_QUIT
void IrcTabItem::onIrcQuit(IrcEvent &ev)
{
    if(m_type == CHANNEL && m_users->findItem(ev.param1) != -1)
    {
        IrcEvent pev;
        pev.eventType = IRC_PART;
        pev.param1 = ev.param1;
        pev.param2 = getText();
        pev.param3 = "";
        pev.param4 = "";
        pev.dccFile = DccFile();
        pev.time = FXSystem::now();
        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &pev);
        removeUser(ev.param1);
        if(ev.param2.empty())
        {
            if(!isCommandIgnored("quit") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s has quit"), ev.param1.text()), 1, ev.time);
        }
        else
        {
            if(!isCommandIgnored("quit") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s has quit (%s)"), ev.param1.text(), ev.param2.text()), 1, ev.time);
        }
    }
    else if(m_type == QUERY && getText() == ev.param1)
    {
        appendIrcStyledText(FXStringFormat(_("%s has quit"), ev.param1.text()), 1, ev.time, FALSE, FALSE);
    }
}

//handle IrcEvent IRC_PART
void IrcTabItem::onIrcPart(IrcEvent &ev)
{
    if(comparecase(ev.param2, getText()) == 0)
    {
        if(ev.param3.empty() && !isCommandIgnored("part") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s has parted %s"), ev.param1.text(), ev.param2.text()), 1, ev.time);
        else if(!isCommandIgnored("part") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s has parted %s (%s)"), ev.param1.text(), ev.param2.text(), ev.param3.text()), 1, ev.time);
        removeUser(ev.param1);
    }
}

//handle IrcEvent IRC_CHNOTICE
void IrcTabItem::onIrcChnotice(IrcEvent &ev)
{
    if(!isCommandIgnored("notice"))
    {
        FXbool tabExist = FALSE;
        for(FXint i = 0; i<m_parent->numChildren(); i+=2)
        {
            if(m_parent->childAtIndex(i)->getMetaClass()==&IrcTabItem::metaClass && m_engine->findTarget(static_cast<IrcTabItem*>(m_parent->childAtIndex(i))))
            {
                if((comparecase(ev.param2, static_cast<IrcTabItem*>(m_parent->childAtIndex(i))->getText()) == 0 && static_cast<IrcTabItem*>(m_parent->childAtIndex(i))->getType() == CHANNEL)
                    || (ev.param1 == static_cast<IrcTabItem*>(m_parent->childAtIndex(i))->getText() && static_cast<IrcTabItem*>(m_parent->childAtIndex(i))->getType() == QUERY && ev.param2 == static_cast<IrcTabItem*>(m_parent->childAtIndex(i))->getNickName()))
                {
                    tabExist = TRUE;
                    break;
                }
            }
        }
        if(tabExist)
        {
            if((comparecase(ev.param2, getText()) == 0 && m_type == CHANNEL) || (ev.param1 == getText() && m_type == QUERY && ev.param2 == getNickName()))
            {
                FXbool needHighlight = FALSE;
                if(ev.param3.contains(getNickName()))
                    needHighlight = highlightNeeded(ev.param3);
                appendIrcStyledText(FXStringFormat(_("%s's NOTICE: %s"), ev.param1.text(), ev.param3.text()), 2, ev.time);
                if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
                {
                    if(needHighlight)
                    {
                        this->setTextColor(preferences.m_highlightColor);
                        if(m_type == CHANNEL) this->setIcon(ICO_CHANNELNEWMSG);
                    }
                    else this->setTextColor(preferences.m_unreadColor);
                    if(m_type == QUERY) this->setIcon(ICO_QUERYNEWMSG);
                }
                if((m_type == CHANNEL && needHighlight) || m_type == QUERY)
                    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_NEWMSG), NULL);
            }
        }
        else
        {
            if(isRightForServerMsg())
            {
                appendIrcStyledText(FXStringFormat(_("%s's NOTICE: %s"), ev.param1.text(), ev.param3.text()), 3, ev.time);
            }
        }
    }
}

//handle IrcEvent IRC_NOTICE
void IrcTabItem::onIrcNotice(IrcEvent &ev)
{
    if(isRightForServerMsg())
    {
        if(ev.param1 == getNickName() && !isCommandIgnored("notice"))
        {
            appendIrcStyledText(FXStringFormat(_("NOTICE for you: %s"), ev.param2.text()), 3, ev.time);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }
        else if(!isCommandIgnored("notice"))
        {
            appendIrcStyledText(FXStringFormat(_("%s's NOTICE: %s"), ev.param1.text(), ev.param2.text()), 3, ev.time);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }
    }
}

//handle IrcEvent IRC_NICK
void IrcTabItem::onIrcNick(IrcEvent &ev)
{
    if(m_users->findItem(ev.param1) != -1)
    {
        if(ev.param2 == getNickName() && !isCommandIgnored("nick")) appendIrcStyledText(FXStringFormat(_("You're now known as %s"), ev.param2.text()), 1, ev.time);
        else if(!isCommandIgnored("nick") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s is now known as %s"), ev.param1.text(), ev.param2.text()), 1, ev.time);
        changeNickUser(ev.param1, ev.param2);
    }
    if(m_type == QUERY && ev.param1 == getText())
    {
        this->setText(ev.param2);
        stopLogging();
    }
}

//handle IrcEvent IRC_TOPIC
void IrcTabItem::onIrcTopic(IrcEvent &ev)
{
    if(comparecase(ev.param2, getText()) == 0)
    {
        appendIrcText(FXStringFormat(_("%s set new topic for %s: %s"), ev.param1.text(), ev.param2.text(), ev.param3.text()), ev.time);
        m_topic = dxutils::stripColors(ev.param3, TRUE);
        m_topicline->setText(m_topic);
        m_topicline->setCursorPos(0);
        m_topicline->makePositionVisible(0);
    }
}

//handle IrcEvent IRC_KICK
void IrcTabItem::onIrcKick(IrcEvent &ev)
{
    if(comparecase(ev.param3, getText()) == 0)
    {
        if(ev.param2 != getNickName())
        {
            if(ev.param4.empty()) appendIrcStyledText(FXStringFormat(_("%s was kicked from %s by %s"), ev.param2.text(), ev.param3.text(), ev.param1.text()), 1, ev.time, FALSE, FALSE);
            else appendIrcStyledText(FXStringFormat(_("%s was kicked from %s by %s (%s)"), ev.param2.text(), ev.param3.text(), ev.param1.text(), ev.param4.text()), 1, ev.time, FALSE, FALSE);
            removeUser(ev.param2);
        }
    }
    if(ev.param2 == getNickName() && isRightForServerMsg())
    {
        if(ev.param4.empty()) appendIrcStyledText(FXStringFormat(_("You were kicked from %s by %s"), ev.param3.text(), ev.param1.text()), 1, ev.time, FALSE, FALSE);
        else appendIrcStyledText(FXStringFormat(_("You were kicked from %s by %s (%s)"), ev.param3.text(), ev.param1.text(), ev.param4.text()), 1, ev.time, FALSE, FALSE);
        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
    }
}

//handle IrcEvent IRC_MODE
void IrcTabItem::onIrcMode(IrcEvent &ev)
{
    if(isRightForServerMsg())
    {
        appendIrcStyledText(FXStringFormat(_("Mode change [%s] for %s"), ev.param1.text(), ev.param2.text()), 1, ev.time, FALSE, FALSE);
        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
    }
}

//handle IrcEvent IRC_UMODE
void IrcTabItem::onIrcUmode(IrcEvent &ev)
{
    FXString moderator = ev.param1;
    FXString channel = ev.param2;
    FXString modes = ev.param3;
    FXString args = ev.param4;
    if(comparecase(channel, getText()) == 0)
    {
        FXbool sign = FALSE;
        int argsiter = 1;
        for(int i =0; i < modes.count(); i++) {
            switch(modes[i]) {
                case '+':
                    sign = TRUE;
                    break;
                case '-':
                    sign = FALSE;
                    break;
                case 'a': //admin
                {
                    FXString nick = dxutils::getParam(args, argsiter, FALSE);
                    setUserMode(nick, sign?ADMIN:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s gave you admin"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s gave %s admin"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s removed you admin"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s removed %s admin"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    argsiter++;
                }break;
                case 'o': //op
                {
                    FXString nick = dxutils::getParam(args, argsiter, FALSE);
                    setUserMode(nick, sign?OP:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s gave you op"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s gave %s op"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s removed you op"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s removed %s op"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    if(getNickName() == nick) sign ? m_iamOp = TRUE : m_iamOp = FALSE;
                    argsiter++;
                }break;
                case 'v': //voice
                {
                    FXString nick = dxutils::getParam(args, argsiter, FALSE);
                    setUserMode(nick, sign?VOICE:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s gave you voice"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s gave %s voice"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s removed you voice"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s removed %s voice"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    argsiter++;
                }break;
                case 'h': //halfop
                {
                    FXString nick = dxutils::getParam(args, argsiter, FALSE);
                    setUserMode(nick, sign?HALFOP:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s gave you halfop"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s gave %s halfop"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, getText()))
                        {
                            if(nick == getNickName()) appendIrcStyledText(FXStringFormat(_("%s removed you halfop"), moderator.text()), 1, ev.time, FALSE, FALSE);
                            else appendIrcStyledText(FXStringFormat(_("%s removed %s halfop"), moderator.text(), nick.text()), 1, ev.time, FALSE, FALSE);
                        }
                    }
                    argsiter++;
                }break;
                case 'b': //ban
                {
                    FXString banmask = dxutils::getParam(args, argsiter, FALSE);
                    onBan(banmask, sign, moderator, ev.time);
                    argsiter++;
                }break;
                case 't': //topic settable by channel operator
                {
                    sign ? m_editableTopic = FALSE : m_editableTopic = TRUE;
                }break;
                default:
                {
                    appendIrcStyledText(FXStringFormat(_("%s set Mode: %s"), moderator.text(), FXString(modes+" "+args).text()), 1, ev.time, FALSE, FALSE);
                }
            }
        }
        if(preferences.m_sortUserMode) m_users->sortItems();
    }
}

//handle IrcEvent IRC_CHMODE
void IrcTabItem::onIrcChmode(IrcEvent &ev)
{
    FXString channel = ev.param1;
    FXString modes = ev.param2;
    if(comparecase(channel, getText()) == 0)
    {
        if(modes.contains('t')) m_editableTopic = FALSE;
        appendIrcStyledText(FXStringFormat(_("Mode for %s: %s"), channel.text(), modes.text()), 1, ev.time, FALSE, FALSE);
    }
}

//handle IrcEvent IRC_SERVERREPLY
void IrcTabItem::onIrcServerReply(IrcEvent &ev)
{
    if(m_type == SERVER)
    {
        //this->setText(server->GetRealServerName());
        appendIrcText(ev.param1, ev.time);
        if(getApp()->getForeColor() == this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
    }
    else
    {
        if(isRightForServerMsg())
        {
            appendIrcText(ev.param1, ev.time, FALSE, FALSE);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }
    }
}

//handle IrcEvent IRC_CONNECT
void IrcTabItem::onIrcConnect(IrcEvent &ev)
{
    appendIrcStyledText(ev.param1, 3, ev.time, FALSE, FALSE);
}

//handle IrcEvent IRC_ERROR
void IrcTabItem::onIrcError(IrcEvent &ev)
{
    appendIrcStyledText(ev.param1, 4, ev.time, FALSE, FALSE);
}

//handle IrcEvent IRC_SERVERERROR
void IrcTabItem::onIrcServerError(IrcEvent &ev)
{
    if(isRightForServerMsg())
    {
        appendIrcStyledText(ev.param1, 4, ev.time, FALSE, FALSE);
        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
    }
}

//handle IrcEvent IRC_DISCONNECT
void IrcTabItem::onIrcDisconnect(IrcEvent &ev)
{
    appendIrcStyledText(ev.param1, 4, ev.time, FALSE, FALSE);
    if(isRightForServerMsg())
    {
        if(m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_highlightColor);
    }
    if(m_type == CHANNEL)
    {
        m_users->clearItems();
        m_iamOp = FALSE;
    }
}

//handle IrcEvent IRC_RECONNECT
void IrcTabItem::onIrcReconnect(IrcEvent &ev)
{
    appendIrcStyledText(ev.param1, 4, ev.time, FALSE, FALSE);
    if(isRightForServerMsg())
    {
        if(m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_highlightColor);
    }
    if(m_type == CHANNEL)
    {
        m_users->clearItems();
        m_iamOp = FALSE;
    }
}

//handle IrcEvent IRC_RECONNECTALL
void IrcTabItem::onIrcReconnectAll(IrcEvent &ev)
{
    appendIrcStyledText(ev.param1, 3, ev.time, FALSE, FALSE);
    if(m_type == CHANNEL)
    {
        m_users->clearItems();
        m_iamOp = FALSE;
    }
}

//handle IrcEvent IRC_UNKNOWN
void IrcTabItem::onIrcUnknown(IrcEvent &ev)
{
    if(isRightForServerMsg())
    {
        appendIrcStyledText(FXStringFormat(_("Unhandled command '%s' params: %s"), ev.param1.text(), ev.param2.text()), 4, ev.time, FALSE, FALSE);
        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
    }
}

//handle IrcEvent IRC_301
void IrcTabItem::onIrc301(IrcEvent &ev)
{
    if(m_parent->getCurrent()*2 == m_parent->indexOfChild(this) || getText() == ev.param1)
    {
        if(!isCommandIgnored("away") && !m_engine->isUserIgnored(ev.param1, getText())) appendIrcStyledText(FXStringFormat(_("%s is away: %s"),ev.param1.text(), ev.param2.text()), 1, ev.time);
    }
}

//handle IrcEvent IRC_305
void IrcTabItem::onIrc305(IrcEvent &ev)
{
    FXint i = m_users->findItem(getNickName());
    if(i != -1)
    {
        appendIrcStyledText(ev.param1, 1, ev.time, FALSE, FALSE);
        ((NickListItem*)m_users->getItem(i))->changeAway(FALSE);
    }
}

//handle IrcEvent IRC_306
void IrcTabItem::onIrc306(IrcEvent &ev)
{
    FXint i = m_users->findItem(getNickName());
    if(i != -1)
    {
        appendIrcStyledText(ev.param1, 1, ev.time, FALSE, FALSE);
        ((NickListItem*)m_users->getItem(i))->changeAway(TRUE);
    }
}

//handle IrcEvent IRC_331, IRC_332 and IRC_333
void IrcTabItem::onIrc331332333(IrcEvent &ev)
{
    if(comparecase(ev.param1, getText()) == 0)
    {
        appendIrcText(ev.param2, ev.time);
        if(ev.eventType == IRC_331)
        {
            m_topic = dxutils::stripColors(ev.param2, TRUE);
            m_topicline->setText(m_topic);
            m_topicline->setCursorPos(0);
            m_topicline->makePositionVisible(0);
        }
        if(ev.eventType == IRC_332)
        {
            m_topic = dxutils::stripColors(dxutils::getParam(ev.param2, 2, TRUE, ':').after(' '), TRUE);
            m_topicline->setText(m_topic);
            m_topicline->setCursorPos(0);
            m_topicline->makePositionVisible(0);
        }
    }
}

//handle IrcEvent IRC_353
void IrcTabItem::onIrc353(IrcEvent &ev)
{
    FXString channel = ev.param1;
    FXString usersStr = ev.param2;
    bool list = ev.param3=="list";
    FXString myNick = getNickName();
    if(usersStr.right(1) != " ") usersStr.append(" ");
    if(comparecase(channel, getText()) == 0)
    {
        if(list && !isCommandIgnored("numeric")) appendIrcText(FXStringFormat(_("Users on %s: %s"), channel.text(), usersStr.text()), ev.time, FALSE, FALSE);
        while(usersStr.contains(' '))
        {
            FXString nick = usersStr.before(' ');
            UserMode mode = getUserMode(nick[0]);
            if(mode != NONE) nick = nick.after(nick[0]);
            addUser(nick, mode);
            if(mode == OP && nick == myNick) m_iamOp = TRUE;
            usersStr = usersStr.after(' ');
        }
    }
    else
    {
        if(list && !isCommandIgnored("numeric"))
        {
            FXbool channelOn = FALSE;
            for(FXint i = 0; i<m_parent->numChildren(); i+=2)
            {
                if(m_engine->findTarget(static_cast<IrcTabItem*>(m_parent->childAtIndex(i))) && comparecase(static_cast<FXTabItem*>(m_parent->childAtIndex(i))->getText(), channel) == 0)
                {
                    channelOn = TRUE;
                    break;
                }
            }
            if(!channelOn)
            {
                if(m_type == SERVER)
                {
                    appendIrcText(FXStringFormat(_("Users on %s: %s"), channel.text(), usersStr.text()), ev.time, FALSE, FALSE);
                    if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
                }
                else
                {
                    if(isRightForServerMsg())
                    {
                        appendIrcText(FXStringFormat(_("Users on %s: %s"), channel.text(), usersStr.text()), ev.time, FALSE, FALSE);
                        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
                    }
                }
            }
        }
    }
}

//handle IrcEvent IRC_366
void IrcTabItem::onIrc366(IrcEvent &ev)
{
    bool list = ev.param3=="list";
    if(comparecase(ev.param1, getText()) == 0)
    {
        if(list && !isCommandIgnored("numeric")) appendIrcText(FXStringFormat("%s: %s", ev.param1.text(), ev.param2.text()), ev.time, FALSE, FALSE);
        else m_engine->addIgnoreWho(getText());
    }
    else
    {
        if(list && !isCommandIgnored("numeric"))
        {
            FXbool channelOn = FALSE;
            for(FXint i = 0; i<m_parent->numChildren(); i+=2)
            {
                if(m_engine->findTarget(static_cast<IrcTabItem*>(m_parent->childAtIndex(i))) && comparecase(static_cast<FXTabItem*>(m_parent->childAtIndex(i))->getText(), ev.param1) == 0)
                {
                    channelOn = TRUE;
                    break;
                }
            }
            if(!channelOn)
            {
                if(m_type == SERVER)
                {
                    appendIrcText(FXStringFormat("%s: %s", ev.param1.text(), ev.param2.text()), ev.time, FALSE, FALSE);
                    if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
                }
                else
                {
                    if(isRightForServerMsg())
                    {
                        appendIrcText(FXStringFormat("%s: %s", ev.param1.text(), ev.param2.text()), ev.time, FALSE, FALSE);
                        if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
                    }
                }
            }
        }
    }
}

//handle IrcEvent IRC_372
void IrcTabItem::onIrc372(IrcEvent &ev)
{
    if(m_type == SERVER)
    {
        appendIrcText(ev.param1, ev.time);
        if(getApp()->getForeColor() == this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
    }
    else
    {
        if(isRightForServerMsg())
        {
            appendIrcText(ev.param1, ev.time, FALSE, FALSE);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }
    }
}

//handle IrcEvent IRC_AWAY
void IrcTabItem::onIrcAway(IrcEvent &ev)
{
    if(comparecase(ev.param1, getText()) == 0)
    {
        onAway();
        return;
    }
    //hack for &bitlbee
    if(comparecase("&bitlbee", getText()) == 0 && m_users->findItem(ev.param1)!=-1)
    {
        onAway();
        return;
    }
}

//handle IrcEvent IRC_ENDMOTD
void IrcTabItem::onIrcEndMotd()
{
    m_text->makeLastRowVisible(TRUE);
    if(m_type == SERVER && !m_engine->getNetworkName().empty())
        setText(m_engine->getNetworkName());
}

long IrcTabItem::onPipe(FXObject*, FXSelector, void *ptr)
{
    FXString text = *(FXString*)ptr;
    if(m_sendPipe && (m_type == CHANNEL || m_type == QUERY))
    {
        if(!getApp()->hasTimeout(this, IrcTabItem_PTIME)) getApp()->addTimeout(this, IrcTabItem_PTIME);
        m_pipeStrings.append(text);
    }
    else appendIrcText(text, FXSystem::now(), TRUE, FALSE);
    return 1;
}

//checking away in channel
void IrcTabItem::checkAway()
{
    if(m_type == CHANNEL && m_engine->getConnected() && m_numberUsers < preferences.m_maxAway)
    {
        m_engine->addIgnoreWho(getText());
    }
}

long IrcTabItem::onPipeTimeout(FXObject*, FXSelector, void*)
{
    if(m_type == CHANNEL || m_type == QUERY)
    {
        if(m_pipeStrings.no() > 3)
        {
            if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
            if(m_pipeStrings[0].length() > m_maxLen-10-getText().length())
            {
                dxStringArray messages = cutText(m_pipeStrings[0], m_maxLen-10-getText().length());
                for(FXint i=0; i<messages.no(); i++)
                {
#ifdef HAVE_LUA
                    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_OWNMSG), &messages[i]);
                    if(!m_scriptHasOwnMsg)
                    {
                        appendMyMsg(messages[i], FXSystem::now());
                        m_engine->sendMsg(getText(), messages[i]);
                    }
#else
                    appendMyMsg(messages[i], FXSystem::now());
                    m_engine->sendMsg(getText(), messages[i]);
#endif
                }
            }
            else
            {
#ifdef HAVE_LUA
                m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_OWNMSG), &m_pipeStrings[0]);
                if(!m_scriptHasOwnMsg)
                {
                    appendMyMsg(m_pipeStrings[0], FXSystem::now());
                    m_engine->sendMsg(getText(), m_pipeStrings[0]);
                }
#else
                appendMyMsg(m_pipeStrings[0], FXSystem::now());
                m_engine->sendMsg(getText(), m_pipeStrings[0]);
#endif
            }
            m_pipeStrings.erase(0);
            getApp()->addTimeout(this, IrcTabItem_PTIME, 3000);
        }
        else
        {
            while(m_pipeStrings.no())
            {
                if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
                if(m_pipeStrings[0].length() > m_maxLen-10-getText().length())
                {
                    dxStringArray messages = cutText(m_pipeStrings[0], m_maxLen-10-getText().length());
                    for(FXint i=0; i<messages.no(); i++)
                    {
#ifdef HAVE_LUA
                        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_OWNMSG), &messages[i]);
                        if(!m_scriptHasOwnMsg)
                        {
                            appendMyMsg(messages[i], FXSystem::now());
                            m_engine->sendMsg(getText(), messages[i]);
                        }
#else
                        appendMyMsg(messages[i], FXSystem::now());
                        m_engine->sendMsg(getText(), messages[i]);
#endif
                    }
                }
                else
                {
#ifdef HAVE_LUA
                    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_OWNMSG), &m_pipeStrings[0]);
                    if(!m_scriptHasOwnMsg)
                    {
                        appendMyMsg(m_pipeStrings[0], FXSystem::now());
                        m_engine->sendMsg(getText(), m_pipeStrings[0]);
                    }
#else
                    appendMyMsg(m_pipeStrings[0], FXSystem::now());
                    m_engine->sendMsg(getText(), m_pipeStrings[0]);
#endif
                }
                m_pipeStrings.erase(0);
            }
        }
    }
    return 1;
}

long IrcTabItem::onEggTimeout(FXObject*, FXSelector, void*)
{
    if(m_pics<24)
    {
        getApp()->addTimeout(this, IrcTabItem_ETIME, 222);
        FXString replace = "\n";
        FXint max = rand()%30;
        for(FXint i=0; i<max; i++)
            replace.append(' ');
        FXString pic = "\n  oooooooooo\n o          o\no            o\no   o    o   o\no   o    o   o\no            o\no  o      o  o\no   oooooo   o\n o          o\n  oooooooooo";
        pic.substitute("\n", replace);
        pic.append('\n');
        m_text->clearText();
        max = rand()%15;
        for(FXint i=0; i<max; i++)
            pic.prepend('\n');
        max = rand()%7;
        m_text->appendStyledText(pic, max+12);
        m_pics++;
    }
    return 1;
}

void IrcTabItem::onAway()
{
    if(m_numberUsers < preferences.m_maxAway)
    {
        for(FXint i = 0; i < m_users->getNumItems(); i++)
        {
            FXbool away = m_engine->getNickInfo(m_users->getItemText(i)).away;
            ((NickListItem*)m_users->getItem(i))->changeAway(away);
        }
    }
    else
    {
        for(FXint i = 0; i < m_users->getNumItems(); i++)
        {
            ((NickListItem*)m_users->getItem(i))->changeAway(FALSE);
        }
    }
}

long IrcTabItem::onRightMouse(FXObject *, FXSelector, void *ptr)
{
    //focus();
    FXEvent* event = (FXEvent*)ptr;
    if(event->moved) return 1;
    FXint index = m_users->getItemAt(event->win_x,event->win_y);
    if(index >= 0)
    {
        UserMode mode = ((NickListItem*)m_users->getItem(index))->getUserMode();
        NickInfo nick = m_engine->getNickInfo(m_users->getItemText(index));
        m_nickOnRight = nick;
        FXString flagpath = DXIRC_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "flags";
        delete ICO_FLAG;
        ICO_FLAG = NULL;
        if(FXStat::exists(flagpath+PATHSEPSTRING+nick.host.rafter('.')+".png")) ICO_FLAG = makeIcon(getApp(), flagpath, nick.host.rafter('.')+".png", TRUE);
        else ICO_FLAG = makeIcon(getApp(), flagpath, "unknown.png", TRUE);
        FXMenuPane opmenu(this);
        if(mode != OP) new FXMenuCommand(&opmenu, _("Give op"), NULL, this, IrcTabItem_OP);
        if(mode == OP) new FXMenuCommand(&opmenu, _("Remove op"), NULL, this, IrcTabItem_DEOP);
        new FXMenuSeparator(&opmenu);
        if(mode != VOICE) new FXMenuCommand(&opmenu, _("Give voice"), NULL, this, IrcTabItem_VOICE);
        if(mode == VOICE) new FXMenuCommand(&opmenu, _("Remove voice"), NULL, this, IrcTabItem_DEVOICE);
        new FXMenuSeparator(&opmenu);
        new FXMenuCommand(&opmenu, _("Kick"), NULL, this, IrcTabItem_KICK);
        new FXMenuSeparator(&opmenu);
        new FXMenuCommand(&opmenu, _("Ban"), NULL, this, IrcTabItem_BAN);
        new FXMenuCommand(&opmenu, _("KickBan"), NULL, this, IrcTabItem_KICKBAN);
        FXMenuPane popup(this);
        new FXMenuCommand(&popup, FXStringFormat(_("User: %s@%s"), nick.user.text(), nick.host.text()), ICO_FLAG);
        new FXMenuCommand(&popup, FXStringFormat(_("Realname: %s"), nick.real.text()));
        if(nick.nick != getNickName())
        {
            new FXMenuSeparator(&popup);
            new FXMenuCommand(&popup, _("Query"), NULL, this, IrcTabItem_NEWQUERY);
            new FXMenuCommand(&popup, _("User information (WHOIS)"), NULL, this, IrcTabItem_WHOIS);
            new FXMenuCommand(&popup, _("DCC chat"), NULL, this, IrcTabItem_DCCCHAT);
            new FXMenuCommand(&popup, _("Send file"), NULL, this, IrcTabItem_DCCSEND);
            new FXMenuCommand(&popup, _("Ignore"), NULL, this, IrcTabItem_IGNORE);
            if(m_iamOp) new FXMenuCascade(&popup, _("Operator actions"), NULL, &opmenu);
        }
        else
        {
            new FXMenuSeparator(&popup);
            if(m_engine->isAway(getNickName())) new FXMenuCommand(&popup, _("Remove Away"), NULL, this, IrcTabItem_DEAWAY);
            else new FXMenuCommand(&popup, _("Set Away"), NULL, this, IrcTabItem_AWAY);
        }
        popup.create();
        popup.popup(NULL,event->root_x,event->root_y);
        getApp()->runModalWhileShown(&popup);
    }
    return 1;
}

long IrcTabItem::onDoubleclick(FXObject*, FXSelector, void*)
{
    FXint index = m_users->getCursorItem();
    if(index >= 0)
    {
        if(m_users->getItemText(index) == getNickName()) return 1;
        IrcEvent ev;
        ev.eventType = IRC_QUERY;
        ev.param1 = m_users->getItemText(index);
        ev.param2 = getNickName();
        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
    }
    return 1;
}

long IrcTabItem::onNewQuery(FXObject *, FXSelector, void *)
{
    IrcEvent ev;
    ev.eventType = IRC_QUERY;
    ev.param1 = m_nickOnRight.nick;
    ev.param2 = getNickName();
    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
    return 1;
}

long IrcTabItem::onWhois(FXObject *, FXSelector, void *)
{
    m_engine->sendWhois(m_nickOnRight.nick);
    return 1;
}

long IrcTabItem::onDccChat(FXObject*, FXSelector, void*)
{
    IrcEvent ev;
    ev.eventType = IRC_DCCSERVER;
    ev.param1 = m_nickOnRight.nick;
    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
    return 1;
}

long IrcTabItem::onDccSend(FXObject*, FXSelector, void*)
{
    DccSendDialog dialog((FXMainWindow*)m_parent->getParent()->getParent(), m_nickOnRight.nick);
    if(dialog.execute())
    {
        IrcEvent ev;
        ev.eventType = dialog.getPassive() ? IRC_DCCPOUT: IRC_DCCOUT;
        ev.param1 = m_nickOnRight.nick;
        ev.param2 = dialog.getFilename();
        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
    }
    return 1;
}

long IrcTabItem::onOp(FXObject *, FXSelector, void *)
{
    m_engine->sendMode(getText()+" +o "+m_nickOnRight.nick);
    return 1;
}

long IrcTabItem::onDeop(FXObject *, FXSelector, void *)
{
    m_engine->sendMode(getText()+" -o "+m_nickOnRight.nick);
    return 1;
}

long IrcTabItem::onVoice(FXObject *, FXSelector, void *)
{
    m_engine->sendMode(getText()+" +v "+m_nickOnRight.nick);
    return 1;
}

long IrcTabItem::onDevoice(FXObject *, FXSelector, void *)
{
    m_engine->sendMode(getText()+" -v "+m_nickOnRight.nick);
    return 1;
}

long IrcTabItem::onKick(FXObject *, FXSelector, void *)
{
    FXDialogBox kickDialog(this, _("Kick dialog"), DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    FXVerticalFrame *contents = new FXVerticalFrame(&kickDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);

    FXHorizontalFrame *kickframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(kickframe, _("Kick reason:"), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXTextField *reasonEdit = new FXTextField(kickframe, 25, NULL, 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &kickDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &kickDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);

    if(kickDialog.execute(PLACEMENT_CURSOR))
    {
        m_engine->sendKick(getText(), m_nickOnRight.nick, reasonEdit->getText());
    }
    return 1;
}

long IrcTabItem::onBan(FXObject *, FXSelector, void *)
{
    FXDialogBox banDialog(this, _("Ban dialog"), DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    FXVerticalFrame *contents = new FXVerticalFrame(&banDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);

    FXHorizontalFrame *banframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(banframe, _("Banmask:"), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXTextField *banEdit = new FXTextField(banframe, 25, NULL, 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    banEdit->setText(m_nickOnRight.nick+"!"+m_nickOnRight.user+"@"+m_nickOnRight.host);

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &banDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &banDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);

    if(banDialog.execute(PLACEMENT_CURSOR))
    {
        m_engine->sendMode(getText()+" +b "+banEdit->getText());
    }
    return 1;
}

long IrcTabItem::onKickban(FXObject *, FXSelector, void *)
{
    FXDialogBox banDialog(this, _("Kick/Ban dialog"), DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    FXVerticalFrame *contents = new FXVerticalFrame(&banDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);

    FXHorizontalFrame *kickframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(kickframe, _("Kick reason:"), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXTextField *reasonEdit = new FXTextField(kickframe, 25, NULL, 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);

    FXHorizontalFrame *banframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(banframe, _("Banmask:"), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXTextField *banEdit = new FXTextField(banframe, 25, NULL, 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    banEdit->setText(m_nickOnRight.nick+"!"+m_nickOnRight.user+"@"+m_nickOnRight.host);

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &banDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &banDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);

    if(banDialog.execute(PLACEMENT_CURSOR))
    {
        m_engine->sendKick(getText(), m_nickOnRight.nick, reasonEdit->getText());
        m_engine->sendMode(getText()+" +b "+banEdit->getText());
    }
    return 1;
}

//handle popup Ignore
long IrcTabItem::onIgnore(FXObject*, FXSelector, void*)
{
    FXDialogBox dialog(this, _("Add ignore user"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    FXVerticalFrame *contents = new FXVerticalFrame(&dialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    new FXLabel(matrix, _("Nick:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *nick = new FXTextField(matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    nick->setText(m_nickOnRight.nick+"!"+m_nickOnRight.user+"@"+m_nickOnRight.host);
    new FXLabel(matrix, _("Channel(s):\tChannels need to be comma separated"), NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *channel = new FXTextField(matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    channel->setText(getText());
    channel->setTipText(_("Channels need to be comma separated"));
    new FXLabel(matrix, _("Server:"), NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *server = new FXTextField(matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    server->setText(getServerName());

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &dialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &dialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);

    if(dialog.execute(PLACEMENT_CURSOR))
    {
        FXString ignoretext = nick->getText()+" "+channel->getText()+" "+server->getText();
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_ADDIUSER), &ignoretext);
    }
    return 1;
}

//handle IrcTabItem_AWAY
long IrcTabItem::onSetAway(FXObject*, FXSelector, void*)
{
    FXDialogBox awayDialog(this, _("Away dialog"), DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    FXVerticalFrame *contents = new FXVerticalFrame(&awayDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);

    FXHorizontalFrame *msgframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(msgframe, _("Message:"), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXTextField *msgEdit = new FXTextField(msgframe, 25, NULL, 0, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    msgEdit->setText(_("away"));

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &awayDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &awayDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);

    if(awayDialog.execute(PLACEMENT_CURSOR))
    {
        m_engine->sendAway(msgEdit->getText().empty() ? _("away"): msgEdit->getText());
    }
    return 1;
}

//handle IrcTabItem_DEAWAY
long IrcTabItem::onRemoveAway(FXObject*, FXSelector, void*)
{
    m_engine->sendAway("");
    return 1;
}

//handle change in spellLang combobox
long IrcTabItem::onSpellLang(FXObject*, FXSelector, void*)
{
    m_commandline->setLanguage(m_spellLangs->getItemText(m_spellLangs->getCurrentItem()));
    m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),m_spellLangs->getItemText(m_spellLangs->getCurrentItem()).text()));
    return 1;
}

//handle LogThread_LIST
long IrcTabItem::onListLogThread(FXObject*, FXSelector, void*)
{
    dxStringArray files = m_logThread->getResult();
    if(!files.no())
    {
        appendIrcStyledText(FXStringFormat(_("No log files for %s-%s."), m_engine->getNetworkName().text(), getText().text()), 4, FXSystem::now(), FALSE, FALSE);
        return 1;
    }
    appendIrcStyledText(_("Log files:"), 7, FXSystem::now(), FALSE, FALSE);
    FXString lastYear = dxutils::getParam(files[0], 1, FALSE, '-');
    FXint lastMonth = FXIntVal(dxutils::getParam(files[0], 2, FALSE, '-'));
    appendIrcStyledText(FXStringFormat("%s:", lastYear.text()), 5, FXSystem::now(), FALSE, FALSE);
    appendIrcStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 5, FXSystem::now(), FALSE, FALSE);
    FXString list = "";
    for(FXint i=0; i<files.no(); i++)
    {
        if(lastYear!=dxutils::getParam(files[i], 1, FALSE, '-'))
        {
            lastYear = dxutils::getParam(files[i], 1, FALSE, '-');
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendIrcText(list.rbefore(','), FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat("%s:", lastYear.text()), 5, FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 5, FXSystem::now(), FALSE, FALSE);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else if(lastMonth!=FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-')))
        {
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendIrcText(list.rbefore(','), FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 5, FXSystem::now(), FALSE, FALSE);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
    }
    if(!list.empty()) appendIrcText(list.rbefore(','), FXSystem::now(), FALSE, FALSE);
    return 1;
}

//handle LogThread_LINES
long IrcTabItem::onLinesLogThread(FXObject*, FXSelector, void*)
{
    m_loadLog = FALSE;
    clearChat();
    dxStringArray lines = m_logThread->getResult();
    for(FXint i=0; i<lines.no(); i++)
    {
        if(lines[i].after(']')[1]=='\014') //log style mark
        {
            if(lines[i].after('\014',2)[0]=='<') //has nick
            {
                m_text->appendStyledText(lines[i].before('\014'), 11); //timestamp
                FXString nick = lines[i].after('<').before('>');
                m_text->appendStyledText(nick+":", getNickStyle(nick));
                appendLinkText(dxutils::stripColors(lines[i].after('>'), TRUE), 11);
            }
            else appendLinkText(dxutils::stripLogMark(dxutils::stripColors(lines[i], TRUE)), 11);
        }
        else
        {
            if(lines[i].after(']')[1]=='<') //has nick
            {
                m_text->appendStyledText(lines[i].before('<'), 11); //timestamp
                FXString nick = lines[i].after('<').before('>');
                m_text->appendStyledText(nick+":", getNickStyle(nick));
                appendLinkText(dxutils::stripColors(lines[i].after('>'), TRUE), 11);
            }
            else appendLinkText(dxutils::stripColors(lines[i], TRUE), 11);
        }
    }
    while(m_storedEvents.no())
    {
        parseIrcEvent(m_storedEvents[0]);
        m_storedEvents.erase(0);
    }
    return 1;
}

long IrcTabItem::onTopic(FXObject*, FXSelector, void*)
{
    if(m_editableTopic || m_iamOp)
    {
        if(m_topicline->getText().length() > m_engine->getTopicLen())
        {
            appendIrcStyledText(FXStringFormat(_("Warning: topic is too long. Max. topic length is %d."), m_engine->getTopicLen()), 4, FXSystem::now(), FALSE, FALSE);
            m_engine->sendTopic(getText(), m_topicline->getText());
            return 1;
        }
        m_engine->sendTopic(getText(), m_topicline->getText());
    }
    else
    {
        m_topicline->setText(m_topic);
        m_topicline->setCursorPos(0);
        m_topicline->makePositionVisible(0);
    }
    return 1;
}

long IrcTabItem::onTopicLink(FXObject*, FXSelector, void *data)
{
    dxutils::launchLink(static_cast<FXchar*>(data));
    return 1;
}

void IrcTabItem::onBan(const FXString &banmask, const FXbool &sign, const FXString &sender, const FXTime &time)
{
    if(sign)
    {
        FXString nicks = m_engine->getBannedNick(banmask);
        FXString myNick = getNickName();
        while(nicks.contains(';'))
        {
            for(FXint i=m_users->getNumItems()-1; i>-1; i--)
            {
                if(nicks.before(';') == m_users->getItemText(i))
                {
                    if(m_users->getItemText(i) == myNick) appendIrcStyledText(FXStringFormat(_("You was banned by %s"), sender.text()), 1, time, FALSE, FALSE);
                    else
                    {
                        if(!isCommandIgnored("ban") && !m_engine->isUserIgnored(m_users->getItemText(i), getText())) appendIrcStyledText(FXStringFormat(_("%s was banned by %s"), m_users->getItemText(i).text(), sender.text()), 1, time, FALSE, FALSE);
                        //RemoveUser(users->getItemText(i));
                    }
                }
            }
            nicks = nicks.after(';');
        }
    }
}

FXString IrcTabItem::getNick(FXint i)
{
    if(m_users->getNumItems()) return m_users->getItemText(i);
    return "";
}

FXint IrcTabItem::getNickStyle(const FXString &nick)
{
    if(preferences.m_coloredNick) return 12+nick.hash()%8; //12 is first colored nick style
    return 5; //bold
}

FXint IrcTabItem::hiliteStyleExist(FXColor foreColor, FXColor backColor, FXuint style)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        if(m_textStyleList[i].normalForeColor == foreColor
                && m_textStyleList[i].normalBackColor == backColor
                && m_textStyleList[i].style == style)
            return i+1;
    }
    return -1;
}

void IrcTabItem::createHiliteStyle(FXColor foreColor, FXColor backColor, FXuint style)
{
    dxHiliteStyle nstyle = {foreColor,backColor,getApp()->getSelforeColor(),getApp()->getSelbackColor(),style,FALSE};
    m_textStyleList.append(nstyle);
    m_text->setHiliteStyles(m_textStyleList.data());
}

dxStringArray IrcTabItem::cutText(FXString text, FXint len)
{
    register FXint previous = 0;
    register FXint s = -1;
    register FXint lw = 0;
    dxStringArray texts;
    for(FXint i = 0; i<text.length(); i++)
    {
        if(text[i] == ' ') s = i;
        if(lw>len)
        {
            if(s!=-1)
            {
                texts.append(text.mid(previous, s));
                i = s+1;
                s = -1;
                previous = i;
            }
            else
            {
                texts.append(text.mid(previous, lw));
                previous = previous+lw;
            }
            lw = 0;
        }
        lw++;
    }
    if(previous<text.length()) texts.append(text.right(text.length()-previous));
    return texts;
}

void IrcTabItem::setCommandFocus()
{
    m_commandline->setFocus();
}

//for "handle" checking, if script contains "all". Send from dxirc.
void IrcTabItem::setHasAllCommand(FXbool hasAll)
{
    m_scriptHasAll = hasAll;
}

//for "handle" checking, if script contains "ownmsg". Send from dxirc.
void IrcTabItem::setHasOwnMsg(FXbool hasOwn)
{
    m_scriptHasOwnMsg = hasOwn;
}

//check need of highlight in msg
FXbool IrcTabItem::highlightNeeded(const FXString &msg)
{
    FXint pos = msg.find(getNickName());
    if(pos==-1) return FALSE;
    FXbool before = TRUE;
    FXbool after = FALSE;
    if(pos) before = isDelimiter(msg[pos-1]);
    if(pos+getNickName().length() == msg.length()) after = TRUE;
    if(pos+getNickName().length() < msg.length()) after = isDelimiter(msg[pos+getNickName().length()]);
    return before && after;
}

//Load lines from logs
void IrcTabItem::loadLogLines()
{
    if(m_type != CHANNEL && m_type != QUERY)
        return;
    if(!m_engine)
        return;
    if(!preferences.m_loadLog)
        return;
    if(m_logThread->running())
    {
        appendStyledText(_("Thread running, try again after some seconds"), 7, false, false, false);
        return;
    }
    appendIrcStyledText(_("Loading log..."), 11, FXSystem::now(), FALSE, FALSE);
    m_loadLog = TRUE;
    m_logThread->setList(false);
    m_logThread->setDirectory(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText());
    m_logThread->start();
}

//List log files for tab
void IrcTabItem::listLogFiles()
{
    if(m_logThread->running())
    {
        appendStyledText(_("Thread running, try again after some seconds"), 7, false, false, false);
        return;
    }
    m_logThread->setList(true);
    m_logThread->setDirectory(preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText());
    m_logThread->start();
}

void IrcTabItem::showLog(const FXString& path)
{
    if(FXStat::exists(path))
    {
        FXString name = FXPath::name(path);
        clearChat();
        appendStyledText(FXStringFormat(_("Log from %s:"), name.text()), 7, FALSE, FALSE, FALSE);
        std::ifstream file(path.text());
        std::string line;
        FXString fxline;
        while(getline(file, line))
        {
            fxline = line.c_str();
            if(fxline.length()>12 && fxline[11]=='\014') //log style mark
            {
                FXint style = FXIntVal(fxline.after('\014').before('\014'));
                m_text->appendText(fxline.before('\014'));
                if(fxline.after('\014',2)[0]=='<') //has nick
                {
                    FXString nick = fxline.after('<').before('>');
                    FXString msg = fxline.after('>');
                    appendLinkText(nick+":"+msg, style);
                }
                else appendLinkText(fxline.after('\014', 2), style);
            }
            else
            {
                if(fxline.length()>12 && fxline[11]=='<') //line with nick
                {
                    m_text->appendText(fxline.before('<'));
                    FXString nick = fxline.after('<').before('>');
                    FXString msg = fxline.after('>').trimBegin();
                    m_text->appendStyledText(nick+": ", getNickStyle(nick));
                    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, FALSE);
                    appendLinkText(msg, 0);
                }
                else appendIrcText(fxline, 0, FALSE, FALSE);
            }
        }
        file.close();
    }
    else
    {
        appendStyledText(FXStringFormat(_("'%s' path doesn't exist"), path.text()), 4, false, false, false);
    }
}

void IrcTabItem::completeHelpCommand()
{
    FXString line = m_commandline->getText();
    for(FXint i = 0; i < dxUtils.commandsNo(); i++)
    {
        if(comparecase(line.after(' '), dxUtils.commandsAt(i)) == 0)
        {
            if((i+1) < dxUtils.commandsNo()) m_commandline->setText("/HELP "+dxUtils.commandsAt(++i));
            else m_commandline->setText("/HELP "+dxUtils.commandsAt(0));
            break;
        }
        else if(comparecase(line.after(' '), dxUtils.commandsAt(i).left(line.after(' ').length())) == 0)
        {
            m_commandline->setText("/HELP "+dxUtils.commandsAt(i));
            break;
        }
    }
}

FXbool IrcTabItem::commandScript(const FXString &commandtext)
{
    LuaRequest lua;
    lua.type = LUA_COMMAND;
    lua.text = commandtext.after('/');
    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_LUA), &lua);
    return TRUE;
}

FXbool IrcTabItem::commandAdmin(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendAdmin(commandtext.after(' '));
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandAway(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(commandtext.after(' ').length() > m_engine->getAwayLen())
        {
            appendIrcStyledText(FXStringFormat(_("Warning: Away message is too long. Max. away message length is %d."), m_engine->getAwayLen()), 4, FXSystem::now(), FALSE, FALSE);
        }
        return m_engine->sendAway(commandtext.after(' '));
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandBanlist(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString channel = commandtext.after(' ');
        if(channel.empty() && m_type == CHANNEL)
        {
            return m_engine->sendBanlist(getText());
        }
        else if(!isChannel(channel) && m_type != CHANNEL)
        {
            appendIrcStyledText(_("/banlist <channel>, shows banlist for channel."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendBanlist(channel);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandBoats(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString to = commandtext.after(' ');
        if(to.empty())
        {
            appendIrcStyledText(_("/boats <nick>, invites someone to a boats game."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendCtcp(to, "BOATS INVITE");
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandConnect(const FXString &commandtext)
{
    if(commandtext.after(' ').empty())
    {
        appendIrcStyledText(_("/connect <server> [port] [nick] [password] [realname] [channels], connects for given server."), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
    else
    {
        ServerInfo srv;
        srv.hostname = commandtext.after(' ').section(' ', 0);
        srv.port = commandtext.after(' ').section(' ', 1).empty() ? 6667 : FXIntVal(commandtext.after(' ').section(' ', 1));
        srv.nick = commandtext.after(' ').section(' ', 2).empty() ? FXSystem::currentUserName() : commandtext.after(' ').section(' ', 2);
        srv.passwd = commandtext.after(' ').section(' ', 3).empty() ? "" : commandtext.after(' ').section(' ', 3);
        srv.realname = commandtext.after(' ').section(' ', 4).empty() ? FXSystem::currentUserName() : commandtext.after(' ').section(' ', 4);
        srv.channels = commandtext.after(' ').section(' ', 5).empty() ? "" : commandtext.after(' ').section(' ', 5);
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_CSERVER), &srv);
        return TRUE;
    }
}

FXbool IrcTabItem::commandCommands()
{
    appendIrcStyledText(dxUtils.availableCommands(), 3, FXSystem::now(), FALSE, FALSE);
    return TRUE;
}

FXbool IrcTabItem::commandCtcp(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString to = commandtext.after(' ').before(' ');
        FXString msg = commandtext.after(' ', 2);
        if(to.empty() || msg.empty())
        {
            appendIrcStyledText(_("/ctcp <nick> <message>, sends a CTCP message to a user."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        if(msg.length() > m_maxLen-12-to.length())
        {
            appendIrcStyledText(FXStringFormat(_("Warning: ctcp message is too long. Max. ctcp message length is %d."), m_maxLen-12-to.length()), 4, FXSystem::now(), FALSE, FALSE);
        }
        return m_engine->sendCtcp(to, msg);
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandCycle(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(m_type == CHANNEL)
        {
            if(isChannel(commandtext.after(' ')))
            {
                FXString channel = commandtext.after(' ').before(' ');
                FXString reason = commandtext.after(' ', 2);
                reason.empty() ? m_engine->sendPart(channel) : m_engine->sendPart(channel, reason);
                return m_engine->sendJoin(channel);
            }
            else
            {
                commandtext.after(' ').empty() ? m_engine->sendPart(getText()) : m_engine->sendPart(getText(), commandtext.after(' '));
                return m_engine->sendJoin(getText());
            }
        }
        else
        {
            if(isChannel(commandtext.after(' ')))
            {
                FXString channel = commandtext.after(' ').before(' ');
                FXString reason = commandtext.after(' ', 2);
                reason.empty() ? m_engine->sendPart(channel) : m_engine->sendPart(channel, reason);
                return m_engine->sendJoin(channel);
            }
            else
            {
                appendIrcStyledText(_("/cycle <channel> [message], leaves and join channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandDcc(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString dccCommand = dxutils::getParam(commandtext, 2, FALSE).lower();
        if(dccCommand == "chat")
        {
            FXString nick = dxutils::getParam(commandtext, 3, FALSE);
            if(!comparecase(nick, "chat"))
            {
                appendIrcStyledText(_("Nick for chat wasn't entered."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!comparecase(nick, getNickName()))
            {
                appendIrcStyledText(_("Chat with yourself isn't good idea."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            IrcEvent ev;
            ev.eventType = IRC_DCCSERVER;
            ev.param1 = nick;
            m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
            return TRUE;
        }
        else if(dccCommand == "send")
        {
            FXString nick = dxutils::getParam(commandtext, 3, FALSE);
            FXString file = dxutils::getParam(commandtext, 4, TRUE);
            if(!comparecase(nick, "send"))
            {
                appendIrcStyledText(_("Nick for sending file wasn't entered."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!comparecase(nick, getNickName()))
            {
                appendIrcStyledText(_("Sending to yourself isn't good idea."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!comparecase(nick, file))
            {
                appendIrcStyledText(_("Filename wasn't entered"), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!FXStat::exists(file))
            {
                appendIrcStyledText(FXStringFormat(_("File '%s' doesn't exist"), file.text()), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            IrcEvent ev;
            ev.eventType = IRC_DCCOUT;
            ev.param1 = nick;
            ev.param2 = file;
            m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
            return TRUE;
        }
        else if(dccCommand == "psend")
        {
            FXString nick = dxutils::getParam(commandtext, 3, FALSE);
            FXString file = dxutils::getParam(commandtext, 4, TRUE);
            if(!comparecase(nick, "psend"))
            {
                appendIrcStyledText(_("Nick for sending file wasn't entered."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!comparecase(nick, getNickName()))
            {
                appendIrcStyledText(_("Sending to yourself isn't good idea."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!comparecase(nick, file))
            {
                appendIrcStyledText(_("Filename wasn't entered"), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            if(!FXStat::exists(file))
            {
                appendIrcStyledText(FXStringFormat(_("File '%s' doesn't exist"), file.text()), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            IrcEvent ev;
            ev.eventType = IRC_DCCPOUT;
            ev.param1 = nick;
            ev.param2 = file;
            m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
            return TRUE;
        }
        else
        {
            appendIrcStyledText(FXStringFormat(_("'%s' isn't dcc command <chat|send|psend>"), dccCommand.text()), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandDeop(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/deop <nicks>, removes operator status from one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && params.after(' ').empty())
            {
                appendIrcStyledText(_("/deop <channel> <nicks>, removes operator status from one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('-', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                FXString channel = getText();
                FXString nicks = params;
                FXString modeparams = dxutils::createModes('-', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('-', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/deop <channel> <nicks>, removes operator status from one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandDevoice(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/devoice <nicks>, removes voice from one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && params.after(' ').empty())
            {
                appendIrcStyledText(_("/devoice <channel> <nicks>, removes voice from one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('-', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                FXString channel = getText();
                FXString nicks = params;
                FXString modeparams = dxutils::createModes('-', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('-', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/devoice <channel> <nicks>, removes voice from one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandDisconnect(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(commandtext.after(' ').empty()) m_engine->disconnect();
        else m_engine->disconnect(commandtext.after(' '));
        return TRUE;
    }
    else
    {
        m_engine->closeConnection(TRUE);
        return TRUE;
    }
}

FXbool IrcTabItem::commandDxirc()
{
    for(FXint i=0; i<5+rand()%5; i++)
    {
        appendIrcText("", FXSystem::now(), FALSE, FALSE);
    }
    appendIrcText("     __  __         _", FXSystem::now(), FALSE, FALSE);
    appendIrcText("  _/__//__/|_      | |       _", FXSystem::now(), FALSE, FALSE);
    appendIrcText(" /_| |_| |/_/|   __| |__  __|_| _ _  ___", FXSystem::now(), FALSE, FALSE);
    appendIrcText(" |_   _   _|/   / _  |\\ \\/ /| || '_)/ __)", FXSystem::now(), FALSE, FALSE);
    appendIrcText(" /_| |_| |/_/| | (_| | |  | | || | | (__", FXSystem::now(), FALSE, FALSE);
    appendIrcText(" |_   _   _|/   \\____|/_/\\_\\|_||_|  \\___)", FXSystem::now(), FALSE, FALSE);
    appendIrcText("   |_|/|_|/     (c) 2008~ David Vachulka", FXSystem::now(), FALSE, FALSE);
    appendIrcText("   http://dxirc.org", FXSystem::now(), FALSE, FALSE);
    for(FXint i=0; i<5+rand()%5; i++)
    {
        appendIrcText("", FXSystem::now(), FALSE, FALSE);
    }
    return TRUE;
}

FXbool IrcTabItem::commandEgg()
{
    m_text->clearText();
    m_text->appendStyledText(FXString("ahoj sem pan Vajíčko.\n"), 3);
    getApp()->addTimeout(this, IrcTabItem_ETIME, 1000);
    m_pics = 0;
    return TRUE;
}

FXbool IrcTabItem::commandExec(const FXString &commandtext)
{
    FXString params = commandtext.after(' ');
    if(params.empty())
    {
        appendIrcStyledText(_("/exec [-o|-c] <command>, executes command, -o sends output to channel/query, -c closes running command."), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
    else
    {
        if(!m_pipe) m_pipe = new dxPipe(getApp(), this);
        m_pipeStrings.clear();
        if(params.before(' ').contains("-o"))
        {
            m_sendPipe = TRUE;
            m_pipe->execCmd(params.after(' '));
        }
        else if(params.before(' ').contains("-c"))
        {
            m_sendPipe = FALSE;
            m_pipeStrings.clear();
            m_pipe->stopCmd();
        }
        else
        {
            m_sendPipe = FALSE;
            m_pipe->execCmd(params);
        }
        return TRUE;
    }
}

FXbool IrcTabItem::commandHelp(const FXString &commandtext)
{
    return showHelp(commandtext.after(' ').lower().trim());
}

FXbool IrcTabItem::commandIgnore(const FXString &commandtext)
{
    FXString ignorecommand = commandtext.after(' ').before(' ');
    FXString ignoretext = commandtext.after(' ').after(' ');
    if(ignorecommand.empty())
    {
        appendIrcStyledText(_("/ignore <list|addcmd|rmcmd|addusr|rmusr> [command|user] [channel] [server]"), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
    if(comparecase(ignorecommand, "list")==0)
    {
        appendIrcStyledText(_("Ignored commands:"), 7, FXSystem::now(), FALSE, FALSE);
        if(preferences.m_ignoreCommandsList.empty()) appendIrcText(_("No ignored commands"), FXSystem::now(), FALSE, FALSE);
        else appendIrcText(preferences.m_ignoreCommandsList.rbefore(';'), FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Ignored users:"), 7, FXSystem::now(), FALSE, FALSE);
        if(!preferences.m_ignoreUsersList.no()) appendIrcText(_("No ignored users"), FXSystem::now(), FALSE, FALSE);
        else
        {
            for(FXint i=0; i<preferences.m_ignoreUsersList.no(); i++)
            {
                appendIrcText(FXStringFormat(_("%s on channel(s): %s and network: %s"), preferences.m_ignoreUsersList[i].nick.text(), preferences.m_ignoreUsersList[i].channel.text(), preferences.m_ignoreUsersList[i].network.text()), FXSystem::now(), FALSE, FALSE);
            }
        }
        return TRUE;
    }
    else if(comparecase(ignorecommand, "addcmd")==0)
    {
        if(ignoretext.empty())
        {
            appendIrcStyledText(_("/ignore addcmd <command>, adds command to ignored commands."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_ADDICOMMAND), &ignoretext);
        return TRUE;
    }
    else if(comparecase(ignorecommand, "rmcmd")==0)
    {
        if(ignoretext.empty())
        {
            appendIrcStyledText(_("/ignore rmcmd <command>, removes command from ignored commands."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_RMICOMMAND), &ignoretext);
        return TRUE;
    }
    else if(comparecase(ignorecommand, "addusr")==0)
    {
        if(ignoretext.empty())
        {
            appendIrcStyledText(_("/ignore addusr <user> [channel] [server], adds user to ignored users."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_ADDIUSER), &ignoretext);
        return TRUE;
    }
    else if(comparecase(ignorecommand, "rmusr")==0)
    {
        if(ignoretext.empty())
        {
            appendIrcStyledText(_("/ignore rmusr <user>, removes user from ignored users."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_RMIUSER), &ignoretext);
        return TRUE;
    }
    else
    {
        appendIrcStyledText(FXStringFormat(_("'%s' isn't <list|addcmd|rmcmd|addusr|rmusr>"), ignorecommand.text()), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
}

FXbool IrcTabItem::commandInvite(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/invite <nick> <channel>, invites someone to a channel."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else if(isChannel(params) && params.after(' ').empty())
        {
            appendIrcStyledText(_("/invite <nick> <channel>, invites someone to a channel."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            FXString nick = params.before(' ');
            FXString channel = params.after(' ');
            return m_engine->sendInvite(nick, channel);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandJoin(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString channel = commandtext.after(' ');
        if(!isChannel(channel))
        {
            appendIrcStyledText(_("/join <channel>, joins a channel."), 4, FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat(_("'%c' isn't valid char for channel."), channel[0]), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendJoin(channel);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandKick(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/kick <nick>, kicks a user from a channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && params.after(' ').empty())
            {
                appendIrcStyledText(_("/kick <channel> <nick>, kicks a user from a channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nick = params.after(' ');
                FXString reason = params.after(' ', 2);
                if(reason.length() > m_engine->getKickLen())
                {
                    appendIrcStyledText(FXStringFormat(_("Warning: reason of kick is too long. Max. reason length is %d."), m_engine->getKickLen()), 4, FXSystem::now(), FALSE, FALSE);
                    return m_engine->sendKick(channel, nick, reason);
                }
                else return m_engine->sendKick(channel, nick, reason);
            }
            else
            {
                FXString channel = getText();
                FXString nick = params.before(' ');
                FXString reason = params.after(' ');
                if(reason.length() > m_engine->getKickLen())
                {
                    appendIrcStyledText(FXStringFormat(_("Warning: reason of kick is too long. Max. reason length is %d."), m_engine->getKickLen()), 4, FXSystem::now(), FALSE, FALSE);
                    return m_engine->sendKick(channel, nick, reason);
                }
                else return m_engine->sendKick(channel, nick, reason);
            }
        }
        else
        {
            if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nick = params.after(' ');
                FXString reason = params.after(' ', 2);
                if(reason.length() > m_engine->getKickLen())
                {
                    appendIrcStyledText(FXStringFormat(_("Warning: reason of kick is too long. Max. reason length is %d."), m_engine->getKickLen()), 4, FXSystem::now(), FALSE, FALSE);
                    return m_engine->sendKick(channel, nick, reason);
                }
                else return m_engine->sendKick(channel, nick, reason);
            }
            else
            {
                appendIrcStyledText(_("/kick <channel> <nick>, kicks a user from a channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandKill(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        FXString nick = params.before(' ');
        FXString reason = params.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/kill <user> [reason], kills a user from the network."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        if(reason.length() > m_maxLen-7-nick.length())
        {
            appendIrcStyledText(FXStringFormat(_("Warning: reason of kill is too long. Max. reason length is %d."), m_maxLen-7-nick.length()), 4, FXSystem::now(), FALSE, FALSE);
        }
        return m_engine->sendKill(nick, reason);
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandList(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendList(commandtext.after(' '));
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandLog(const FXString &commandtext)
{
    if(m_type == SERVER)
    {
        appendIrcStyledText(_("Server tab hasn't log files"), 4, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    FXString logcommand = commandtext.after(' ').before(' ');
    FXString logfile = commandtext.after(' ').after(' ');
    if(comparecase(logcommand, "list")==0)
    {
        listLogFiles();
        return TRUE;
    }
    else if(comparecase(logcommand, "show")==0)
    {
        if(logfile.empty())
        {
            logfile = FXSystem::time("%Y-%m-%d", FXSystem::now());
        }
        FXString logpath = preferences.m_logPath+PATHSEPSTRING+m_engine->getNetworkName()+PATHSEPSTRING+getText()+PATHSEPSTRING+logfile;
        if(!FXStat::exists(logpath))
        {
            appendIrcStyledText(FXStringFormat(_("Log file %s doesn't exist"), logfile.text()), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_SHOWLOG), &logpath);
        return TRUE;
    }
    else
    {
        appendIrcStyledText(FXStringFormat(_("'%s' isn't <list|show>"), logcommand.text()), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
}

FXbool IrcTabItem::commandLua(const FXString &commandtext)
{
#ifdef HAVE_LUA
    FXString luacommand = commandtext.after(' ').before(' ');
    FXString luatext = commandtext.after(' ').after(' ');
    LuaRequest lua;
    if(luacommand.empty())
    {
        appendIrcStyledText(_("/lua <help|load|unload|list> [scriptpath|scriptname]"), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
    if(comparecase(luacommand, "help")==0)
    {
        appendIrcStyledText(FXStringFormat(_("For help about Lua scripting visit: %s"), LUA_HELP_PATH), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    else if(comparecase(luacommand, "load")==0)
    {
        lua.type = LUA_LOAD;
    }
    else if(comparecase(luacommand, "unload")==0)
    {
        lua.type = LUA_UNLOAD;
    }
    else if(comparecase(luacommand, "list")==0)
    {
        lua.type = LUA_LIST;
    }
    else
    {
        appendIrcStyledText(FXStringFormat(_("'%s' isn't <help|load|unload|list>"), luacommand.text()), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
    }
    lua.text = luatext;
    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_LUA), &lua);
    return TRUE;
#else
    appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, FXSystem::now(), FALSE, FALSE);
    return FALSE;
#endif
}

FXbool IrcTabItem::commandMe(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/me <message>, sends the action to the current channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString message = params.after(' ');
                if(message.length() > m_maxLen-19-channel.length())
                {
                    dxStringArray messages = cutText(message, m_maxLen-19-channel.length());
                    FXbool result = TRUE;
                    for(FXint i=0; i<messages.no(); i++)
                    {
                        if(channel == getText())
                        {
                            appendIrcStyledText(getNickName()+" "+messages[i], 2, FXSystem::now());
                            IrcEvent ev;
                            ev.eventType = IRC_ACTION;
                            ev.param1 = getNickName();
                            ev.param2 = channel;
                            ev.param3 = messages[i];
                            m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                        }
                        result = m_engine->sendMe(channel, messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    if(channel == getText())
                    {
                        appendIrcStyledText(getNickName()+" "+message, 2, FXSystem::now());
                        IrcEvent ev;
                        ev.eventType = IRC_ACTION;
                        ev.param1 = getNickName();
                        ev.param2 = channel;
                        ev.param3 = message;
                        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                    }
                    return m_engine->sendMe(channel, message);
                }
            }
            else
            {
                if(params.length() > m_maxLen-19-getText().length())
                {
                    dxStringArray messages = cutText(params, m_maxLen-19-getText().length());
                    FXbool result = TRUE;
                    for(FXint i=0; i<messages.no(); i++)
                    {
                        appendIrcStyledText(getNickName()+" "+messages[i], 2, FXSystem::now());
                        IrcEvent ev;
                        ev.eventType = IRC_ACTION;
                        ev.param1 = getNickName();
                        ev.param2 = getText();
                        ev.param3 = messages[i];
                        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                        result = m_engine->sendMe(getText(), messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    appendIrcStyledText(getNickName()+" "+params, 2, FXSystem::now());
                    IrcEvent ev;
                    ev.eventType = IRC_ACTION;
                    ev.param1 = getNickName();
                    ev.param2 = getText();
                    ev.param3 = params;
                    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                    return m_engine->sendMe(getText(), params);
                }
            }
        }
        else if(m_type == QUERY)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/me <message>, sends the action to the current query."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else
            {
                if(params.length() > m_maxLen-19-getText().length())
                {
                    dxStringArray messages = cutText(params, m_maxLen-19-getText().length());
                    FXbool result = TRUE;
                    for(FXint i=0; i<messages.no(); i++)
                    {
                        appendIrcStyledText(getNickName()+" "+messages[i], 2, FXSystem::now());
                        IrcEvent ev;
                        ev.eventType = IRC_ACTION;
                        ev.param1 = getNickName();
                        ev.param2 = getText();
                        ev.param3 = messages[i];
                        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                        result = m_engine->sendMe(getText(), messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    appendIrcStyledText(getNickName()+" "+params, 2, FXSystem::now());
                    IrcEvent ev;
                    ev.eventType = IRC_ACTION;
                    ev.param1 = getNickName();
                    ev.param2 = getText();
                    ev.param3 = params;
                    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                    return m_engine->sendMe(getText(), params);
                }
            }
        }
        else
        {
            if(!params.after(' ').empty())
            {
                FXString to = params.before(' ');
                FXString message = params.after(' ');
                if(message.length() > m_maxLen-19-to.length())
                {
                    dxStringArray messages = cutText(message, m_maxLen-19-to.length());
                    FXbool result = TRUE;
                    for(FXint i=0; i<messages.no(); i++)
                    {
                        result = m_engine->sendMe(to, messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    return m_engine->sendMe(to, message);
                }
            }
            else
            {
                appendIrcStyledText(_("/me <to> <message>, sends the action."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandMode(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/mode <channel> <modes>, sets modes for a channel."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendMode(params);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandMsg(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        FXString to = params.before(' ');
        FXString message = params.after(' ');
        if(!to.empty() && !message.empty())
        {
            if(message.length() > m_maxLen-10-to.length())
            {
                dxStringArray messages = cutText(message, m_maxLen-10-to.length());
                FXbool result = TRUE;
                for(FXint i=0; i<messages.no(); i++)
                {
                    if(to == getText())
                    {
                        appendIrcNickText(getNickName(), messages[i], getNickStyle(getNickName()), FXSystem::now());
                        IrcEvent ev;
                        ev.eventType = IRC_PRIVMSG;
                        ev.param1 = getNickName();
                        ev.param2 = to;
                        ev.param3 = messages[i];
                        m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                    }
                    result = m_engine->sendMsg(to, messages[i]) &result;
                }
                return result;
            }
            else
            {
                if(to == getText())
                {
                    appendIrcNickText(getNickName(), message, getNickStyle(getNickName()), FXSystem::now());
                    IrcEvent ev;
                    ev.eventType = IRC_PRIVMSG;
                    ev.param1 = getNickName();
                    ev.param2 = to;
                    ev.param3 = message;
                    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                }
                return m_engine->sendMsg(to, message);
            }
        }
        else
        {
            appendIrcStyledText(_("/msg <nick/channel> <message>, sends a normal message."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandNames(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                return m_engine->sendNames(getText(), true);
            }
            else
            {
                return m_engine->sendNames(params, true);
            }
        }
        else
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/names <channel>, for nicks on a channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else
            {
                return m_engine->sendNames(params, true);
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandNick(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString nick = commandtext.after(' ');
        if(nick.empty())
        {
            appendIrcStyledText(_("/nick <nick>, changes nick."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else if(nick.length() > m_engine->getNickLen())
        {
            appendIrcStyledText(FXStringFormat(_("Warning: nick is too long. Max. nick length is %d."), m_engine->getNickLen()), 4, FXSystem::now(), FALSE, FALSE);
            return m_engine->sendNick(nick);
        }
        else
        {
            return m_engine->sendNick(nick);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandNotice(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        FXString to = params.before(' ');
        FXString message = params.after(' ');
        if(!to.empty() && !message.empty())
        {
            if(message.length() > m_maxLen-9-to.length())
            {
                dxStringArray messages = cutText(message, m_maxLen-9-to.length());
                FXbool result = TRUE;
                for(FXint i=0; i<messages.no(); i++)
                {
                    appendIrcStyledText(FXStringFormat(_("NOTICE to %s: %s"), to.text(), messages[i].text()), 2, FXSystem::now());
                    result = m_engine->sendNotice(to, messages[i]) &result;
                }
                return result;
            }
            else
            {
                appendIrcStyledText(FXStringFormat(_("NOTICE to %s: %s"), to.text(), message.text()), 2, FXSystem::now());
                return m_engine->sendNotice(to, message);
            }
        }
        else
        {
            appendIrcStyledText(_("/notice <nick/channel> <message>, sends a notice."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandOp(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/op <nicks>, gives operator status for one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && params.after(' ').empty())
            {
                appendIrcStyledText(_("/op <channel> <nicks>, gives operator status for one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('+', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                FXString channel = getText();
                FXString nicks = params;
                FXString modeparams = dxutils::createModes('+', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('+', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/op <channel> <nicks>, gives operator status for one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandOper(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        FXString login = params.before(' ');
        FXString password = params.after(' ');
        if(!login.empty() && !password.empty())
        {
            return m_engine->sendOper(login, password);
        }
        else
        {
            appendIrcStyledText(_("/oper <login> <password>, oper up."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandPart(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(m_type == CHANNEL)
        {
            if(commandtext.after(' ').empty())
            {
                return m_engine->sendPart(getText());
            }
            else
            {
                return m_engine->sendPart(getText(), commandtext.after(' '));
            }
        }
        else
        {
            if(isChannel(commandtext.after(' ')))
            {
                FXString channel = commandtext.after(' ').before(' ');
                FXString reason = commandtext.after(' ', 2);
                if(reason.empty())
                {
                    return m_engine->sendPart(channel);
                }
                else
                {
                    return m_engine->sendPart(channel, reason);
                }
            }
            else
            {
                appendIrcStyledText(_("/part <channel> [reason], leaves channel."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandQuery(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(commandtext.after(' ').empty())
        {
            appendIrcStyledText(_("/query <nick>, opens query with nick."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            IrcEvent ev;
            ev.eventType = IRC_QUERY;
            ev.param1 = commandtext.after(' ').before(' ');
            ev.param2 = getNickName();
            m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
            return TRUE;
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandQuit()
{
    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_CQUIT), NULL);
    return TRUE;
}

FXbool IrcTabItem::commandQuote(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendQuote(commandtext.after(' '));
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandSay(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        if (m_type != SERVER && !commandtext.after(' ').empty())
        {
            if(commandtext.after(' ').length() > m_maxLen-10-getText().length())
            {
                dxStringArray messages = cutText(commandtext.after(' '), m_maxLen-10-getText().length());
                FXbool result = TRUE;
                for(FXint i=0; i<messages.no(); i++)
                {
                    appendMyMsg(messages[i], FXSystem::now());
                    IrcEvent ev;
                    ev.eventType = IRC_PRIVMSG;
                    ev.param1 = getNickName();
                    ev.param2 = getText();
                    ev.param3 = messages[i];
                    m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                    result = m_engine->sendMsg(getText(), messages[i]) &result;
                }
                return result;
            }
            else
            {
                appendMyMsg(commandtext.after(' '), FXSystem::now());
                IrcEvent ev;
                ev.eventType = IRC_PRIVMSG;
                ev.param1 = getNickName();
                ev.param2 = getText();
                ev.param3 = commandtext.after(' ');
                m_parent->getParent()->getParent()->handle(m_engine, FXSEL(SEL_COMMAND, IrcEngine_SERVER), &ev);
                return m_engine->sendMsg(getText(), commandtext.after(' '));
            }
        }
        return FALSE;
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandStats(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendStats(commandtext.after(' '));
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandTetris()
{
    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_NEWTETRIS), NULL);
    return TRUE;
}

FXbool IrcTabItem::commandTime()
{
    if(m_engine->getConnected())
    {
        return m_engine->sendQuote("TIME");
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandTopic(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                return m_engine->sendTopic(getText());
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString topic = params.after(' ');
                if(topic.length() > m_engine->getTopicLen())
                {
                    appendIrcStyledText(FXStringFormat(_("Warning: topic is too long. Max. topic length is %d."), m_engine->getTopicLen()), 4, FXSystem::now(), FALSE, FALSE);
                }
                return m_engine->sendTopic(channel, topic);
            }
            else
            {
                if(params.length() > m_engine->getTopicLen())
                {
                    appendIrcStyledText(FXStringFormat(_("Warning: topic is too long. Max. topic length is %d."), m_engine->getTopicLen()), 4, FXSystem::now(), FALSE, FALSE);
                }
                return m_engine->sendTopic(getText(), params);
            }
        }
        else
        {
            if(isChannel(params))
            {
                FXString channel = params.before(' ');
                FXString topic = params.after(' ');
                if(topic.length() > m_engine->getTopicLen())
                {
                    appendIrcStyledText(FXStringFormat(_("Warning: topic is too long. Max. topic length is %d."), m_engine->getTopicLen()), 4, FXSystem::now(), FALSE, FALSE);
                }
                return m_engine->sendTopic(channel, topic);
            }
            else
            {
                appendIrcStyledText(_("/topic <channel> [topic], views or changes channel topic."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandVoice(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(m_type == CHANNEL)
        {
            if(params.empty())
            {
                appendIrcStyledText(_("/voice <nicks>, gives voice for one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && params.after(' ').empty())
            {
                appendIrcStyledText(_("/voice <channel> <nicks>, gives voice for one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            else if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('+', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                FXString channel = getText();
                FXString nicks = params;
                FXString modeparams = dxutils::createModes('+', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.after(' ').empty())
            {
                FXString channel = params.before(' ');
                FXString nicks = params.after(' ');
                FXString modeparams = dxutils::createModes('+', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/voice <channel> <nicks>, gives voice for one or more nicks."), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandWallops(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/wallops <message>, sends wallop message."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            if(params.length() > m_maxLen-9)
            {
                dxStringArray messages = cutText(params, m_maxLen-9);
                FXbool result = TRUE;
                for(FXint i=0; i<messages.no(); i++)
                {
                    result = m_engine->sendWallops(messages[i]) &result;
                }
                return result;
            }
            else return m_engine->sendWallops(params);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandWho(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/who <mask> [o], searchs for mask on network, if o is supplied, only search for opers."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendWho(params);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandWhoami()
{
    if(m_engine->getConnected())
    {
        return m_engine->sendWhoami();
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandWhois(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            if(m_type == QUERY)
            {
                return m_engine->sendWhois(getText());
            }
            appendIrcStyledText(_("/whois <nick>, whois nick."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendWhois(params);
        }
    }
    else
    {
        return notConnected();
    }
}

FXbool IrcTabItem::commandWhowas(const FXString &commandtext)
{
    if(m_engine->getConnected())
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/whowas <nick>, whowas nick."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            return m_engine->sendWhowas(params);
        }
    }
    else
    {
        return notConnected();
    }
}

//fire request for connect dialog
FXbool IrcTabItem::notConnected()
{
    appendIrcStyledText(_("You aren't connected"), 4, FXSystem::now(), FALSE, FALSE);
    m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_CDIALOG), NULL);
    return TRUE;
}

