/*
 *      helpdialog.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "helpdialog.h"
#include "config.h"
#include "i18n.h"
#include "help.h"
#include "dxutils.h"

HelpDialog::HelpDialog(QWidget *parent)
        : QDialog(parent)
{
    m_licenseShowed = false;

    QVBoxLayout *contents = new QVBoxLayout;
    m_text = new QTextEdit;
    m_text->setReadOnly(true);
    m_text->setText(HELP_TEXT);
    contents->addWidget(m_text);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    m_buttonViewLicense  = new QPushButton(_("View license"));
    connect(m_buttonViewLicense, SIGNAL(clicked()), this, SLOT(slotViewLicense()));
    buttons->addButton(m_buttonViewLicense, QDialogButtonBox::ActionRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    contents->addWidget(buttons);
    setLayout(contents);
    setWindowTitle(_("Help"));
    resize(600,400);
}

void HelpDialog::slotViewLicense()
{
    if(m_licenseShowed)
    {
        m_buttonViewLicense->setText(_("View license"));
        m_text->setText(HELP_TEXT);
    }
    else
    {
        m_buttonViewLicense->setText(_("Hide license"));
        if(QFile::exists(dxUtils.getDataPath()+"/COPYING"))
            loadFile(dxUtils.getDataPath()+"/COPYING");
        else
            m_text->setPlainText(_("Unable load COPYING file"));
    }
    m_licenseShowed = !m_licenseShowed;
}

// Load file thanks for xfe again
void HelpDialog::loadFile(const QString& path)
{
    QFile file(path);
    if(file.exists())
    {
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
        {
            m_text->setPlainText(_("Unable to open file: %1").arg(path));
            return;
        }
#ifndef QT_NO_CURSOR
        QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
        m_text->clear();
        while(!file.atEnd())
        {
            m_text->append(QString::fromUtf8(file.readLine()).section('\n',0,0));
        }
        file.close();
#ifndef QT_NO_CURSOR
        QApplication::restoreOverrideCursor();
#endif
    }
    else
    {
        m_text->setPlainText(_("'%1' path doesn't exist").arg(path));
    }
}



