/*
 *      scriptdialog.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef SCRIPTDIALOG_H
#define SCRIPTDIALOG_H

#include <QtGui>
#if(QT_VERSION > 0x050000)
#include <QtWidgets>
#endif
#include <QSyntaxHighlighter>
#include <QHash>
#include <QTextCharFormat>

class ScriptHighlighter : public QSyntaxHighlighter
{
    Q_OBJECT
public:
    ScriptHighlighter(QTextDocument *parent = 0);
protected:
    void highlightBlock(const QString &text);
private:
    struct HighlightingRule
    {
        QRegExp pattern;
        QTextCharFormat format;
    };
    QVector<HighlightingRule> m_highlightingRules;
    QTextCharFormat m_keywordFormat;
    QTextCharFormat m_singleLineCommentFormat;
    QTextCharFormat m_quotationFormat;
    QTextCharFormat m_functionFormat;
};

class ScriptViewDialog: public QDialog
{
    Q_OBJECT
public:
    ScriptViewDialog(const QString &file, QWidget *parent=0);
private:
    QTextEdit *m_editor;
    ScriptHighlighter *m_highlighter;
};

class ScriptDialog: public QDialog
{
    Q_OBJECT
public:
    ScriptDialog(QWidget *parent=0);
private slots:
    void slotLoad();
    void slotView();
    void slotViewNew();
    void slotUnload();
    void slotList();
private:
    QListWidget *m_names;
    QLabel *m_name, *m_version;
    QLabel *m_description, *m_path;
    QPushButton *m_buttonView, *m_buttonUnload;

    void updateList();
    void updateDetails();
    void viewFile(const QString&);
};

#endif // SCRIPTDIALOG_H
