/*
 *      boatstabitem.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "boatstabitem.h"
#include "config.h"
#include "i18n.h"

#define SPACE 5
#define LINEWIDTH 1

Boats::Boats(bool first, QWidget *parent)
    : QWidget(parent), m_first(first), m_placeBoats(true), m_rotated(false), m_previousX(-1), m_previousY(-1),
      m_cells(1), m_onFire(false), m_readyToPlay(false)
{
    setFocusPolicy(Qt::StrongFocus);
    setFocus();
    setMouseTracking(true);
    m_cell = qMin((width()-9*SPACE)/(2*bcolumns), (height()-2*SPACE)/brows);
    clearCellsAndBoats();
    placeMyBoats();
}

void Boats::setCellType(int x, int y, bool my, CELLTYPE type)
{
    if(x>-1 && x<bcolumns && y>-1 && y<brows)
    {
        if(my) m_mycells[x][y].type = type;
        else m_hiscells[x][y].type = type;
    }
}

CELLTYPE Boats::getCellType(int x, int y, bool my)
{
    if(x>-1 && x<bcolumns && y>-1 && y<brows)
    {
        if(my) return m_mycells[x][y].type;
        else return m_hiscells[x][y].type;
    }
    return CLEAR;
}

void Boats::mousePressEvent(QMouseEvent *ev)
{
    if(ev->button() == Qt::LeftButton)
    {
        if(m_placeBoats)
        {
            if(ev->x() < SPACE || ev->x() > SPACE+bcolumns*m_cell)
                return;
            if(ev->y() < SPACE || ev->y() > SPACE+brows*m_cell)
                return;
            int x = (ev->x()-SPACE)/m_cell;
            int y = (ev->y()-SPACE)/m_cell;
            if(x>-1 && x<bcolumns && y>-1 && y<brows
                    && m_mycells[x][y].type == PLACED)
            {
                m_previousX = -1;
                m_previousY = -1;
                for(int i=0; i<m_cells; i++)
                {
                    if(m_rotated) m_mycells[x][y+i].type = BOAT;
                    else m_mycells[x+i][y].type = BOAT;
                }
                for(int i=0; i<boats; i++)
                {
                    if(!m_boats[i].isValid())
                    {
                        m_boats[i].x = x;
                        m_boats[i].y = y;
                        m_boats[i].cells = m_cells;
                        m_boats[i].rotated = m_rotated;
                        break;
                    }
                }
            }
            update();
            placeMyBoats();
        }
        else
        {
            if(m_onFire)
            {
                if(ev->x() < 6*SPACE+bcolumns*m_cell || ev->x() > 6*SPACE+2*bcolumns*m_cell)
                    return;
                if(ev->y() < SPACE || ev->y() > SPACE+brows*m_cell)
                    return;
                int x = (ev->x()-6*SPACE-bcolumns*m_cell)/m_cell;
                int y = (ev->y()-SPACE)/m_cell;
                if(x>-1 && x<bcolumns && y>-1 && y<brows
                        && m_hiscells[x][y].type == CLEAR)
                {
                    m_hiscells[x][y].type = FIRE;
                    emit fire(x, y);
                    m_onFire = false;
                }
                update();
            }
        }
    }
    if(ev->button() == Qt::RightButton)
    {
        if(m_placeBoats)
        {
            if(ev->x() < SPACE || ev->x() > SPACE+bcolumns*m_cell)
                return;
            if(ev->y() < SPACE || ev->y() > SPACE+brows*m_cell)
                return;
            int x = (ev->x()-SPACE)/m_cell;
            int y = (ev->y()-SPACE)/m_cell;
            if(x>-1 && x<bcolumns && y>-1 && y<brows && isRightForPlaceBoat(x, y))
            {
                for(int i=0; i<m_cells; i++)
                {
                    if(m_rotated) m_mycells[x][y+i].type = CLEAR;
                    else m_mycells[x+i][y].type = CLEAR;
                }
            }
            m_rotated = !m_rotated;
            if(x>-1 && x<bcolumns && y>-1 && y<brows && isRightForPlaceBoat(x, y))
            {
                for(int i=0; i<m_cells; i++)
                {
                    if(m_rotated) m_mycells[x][y+i].type = PLACED;
                    else m_mycells[x+i][y].type = PLACED;
                }
            }
            update();
        }
    }
}

void Boats::mouseMoveEvent(QMouseEvent *ev)
{
    if(m_placeBoats)
    {
        if(ev->x() < SPACE || ev->x() > SPACE+bcolumns*m_cell)
            return;
        if(ev->y() < SPACE || ev->y() > SPACE+brows*m_cell)
            return;
        int x = (ev->x()-SPACE)/m_cell;
        int y = (ev->y()-SPACE)/m_cell;
        if((m_previousX!=x || m_previousY!=y)
                && m_previousX>-1 && m_previousX<bcolumns
                && m_previousY>-1 && m_previousY<brows)
        {
            for(int i=0; i<m_cells; i++)
            {
                if(m_rotated)
                {
                    if(m_mycells[m_previousX][m_previousY+i].type == PLACED) m_mycells[m_previousX][m_previousY+i].type = CLEAR;
                }
                else
                {
                    if(m_mycells[m_previousX+i][m_previousY].type == PLACED) m_mycells[m_previousX+i][m_previousY].type = CLEAR;
                }
            }
        }
        if(x>-1 && x<bcolumns && y>-1 && y<brows && isRightForPlaceBoat(x, y))
        {
            m_previousX = x;
            m_previousY = y;
            for(int i=0; i<m_cells; i++)
            {
                if(m_rotated) m_mycells[x][y+i].type = PLACED;
                else m_mycells[x+i][y].type = PLACED;
            }
        }
        update();
    }
}

void Boats::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.fillRect(QRect(0, 0, width(), height()), QColor("black"));
    painter.setPen(QPen(QColor(168,168,168),LINEWIDTH));
    int x1 = 0;
    int y1 = 0;
    for(int x=0; x<bcolumns; ++x)
    {
        for(int y=0; y<brows; ++y)
        {
            painter.drawRect(SPACE+x1,SPACE+y1, m_cell, m_cell);
            painter.drawRect(6*SPACE+bcolumns*m_cell+x1,SPACE+y1, m_cell, m_cell);
            y1 += m_cell;
        }
        y1 = 0;
        x1 += m_cell;
    }
    for(int x=0; x<bcolumns; ++x)
    {
        for(int y=0; y<brows; ++y)
        {
            painter.setBrush(getCellColor(m_mycells[x][y].type));
            painter.drawRect(SPACE+x*m_cell+1, SPACE+y*m_cell+1, m_cell-LINEWIDTH, m_cell-LINEWIDTH);
            painter.setBrush(getCellColor(m_hiscells[x][y].type));
            painter.drawRect(6*SPACE+bcolumns*m_cell+x*m_cell+1, SPACE+y*m_cell+1, m_cell-LINEWIDTH, m_cell-LINEWIDTH);
        }
    }
}

void Boats::resizeEvent(QResizeEvent *event)
{
    m_cell = qMin((event->size().width()-9*SPACE)/(2*bcolumns), (event->size().height()-2*SPACE)/brows);
}

bool Boats::placeBoats() const
{
    return m_placeBoats;
}

void Boats::setPlaceBoats(bool placeBoats)
{
    m_placeBoats = placeBoats;
}

bool Boats::readyToPlay() const
{
    return m_readyToPlay;
}

void Boats::setReadyToPlay(bool readyToPlay)
{
    m_readyToPlay = readyToPlay;
}

bool Boats::onFire() const
{
    return m_onFire;
}

void Boats::setOnFire(bool onFire)
{
    m_onFire = onFire;
}

bool Boats::first() const
{
    return m_first;
}

void Boats::setFirst(bool first)
{
    m_first = first;
}

void Boats::placeMyBoats()
{
    int cells = 0;
    for(int x=0; x<bcolumns; ++x)
    {
        for(int y=0; y<brows; ++y)
        {
            if(m_mycells[x][y].type == BOAT)
                cells++;
        }
    }
    if(cells < 4)
    {
        m_cells = 1;
        return;
    }
    if(cells < 10)
    {
        m_cells = 2;
        return;
    }
    if(cells < 16)
    {
        m_cells = 3;
        return;
    }
    if(cells == 16)
    {
        m_cells = 4;
        return;
    }
    if(cells == 20)
    {
        m_placeBoats = false;
        emit statusText(_("Placing of boats finished"));
        emit ready();
        return;
    }
}

QColor Boats::getCellColor(CELLTYPE type)
{
    switch (type) {
    case CLEAR: return QColor(0,0,0);
    case BOAT: return QColor(78,154,6);
    case WATER: return QColor(32,74,135);
    case HIT: return QColor(164,0,0);
    case PLACED: return QColor(78,154,6);
    case FIRE: return QColor(196,160,0);
    default: return QColor(0,0,0);
    }
}

bool Boats::isRightForPlaceBoat(int x, int y)
{
    if(m_rotated)
    {
        if(y+m_cells-1 < brows-1 && m_mycells[x][y+m_cells].type == BOAT)
        {
            return false;
        }
        for(int i=0; i<m_cells; i++)
        {
            if(y+i > brows-1)
            {
                return false;
            }
            if(m_mycells[x][y+i].type == BOAT)
            {
                return false;
            }
            if(x > 0 && m_mycells[x-1][y+i].type == BOAT)
            {
                return false;
            }
            if(x < bcolumns-1 && m_mycells[x+1][y+i].type == BOAT)
            {
                return false;
            }
            if(y > 0 && m_mycells[x][y-1].type == BOAT)
            {
                return false;
            }
            if(y < brows-1 && m_mycells[x][y+1].type == BOAT)
            {
                return false;
            }
        }
    }
    else
    {
        if(x+m_cells-1 < bcolumns-1 && m_mycells[x+m_cells][y].type == BOAT)
        {
            return false;
        }
        for(int i=0; i<m_cells; i++)
        {
            if(x+i > bcolumns-1)
            {
                return false;
            }
            if(m_mycells[x+i][y].type == BOAT)
            {
                return false;
            }
            if(x > 0 && m_mycells[x-1][y].type == BOAT)
            {
                return false;
            }
            if(x < bcolumns-1 && m_mycells[x+1][y].type == BOAT)
            {
                return false;
            }
            if(y > 0 && m_mycells[x+i][y-1].type == BOAT)
            {
                return false;
            }
            if(y < brows-1 && m_mycells[x+i][y+1].type == BOAT)
            {
                return false;
            }
        }
    }
    return true;
}

void Boats::clearCellsAndBoats()
{
    for(int x=0; x<bcolumns; ++x)
    {
        for(int y=0; y<brows; ++y)
        {
            m_mycells[x][y].type = CLEAR;
            m_hiscells[x][y].type = CLEAR;
        }
    }
    for(int i=0; i<boats; ++i)
    {
        m_boats[i].x = -1;
        m_boats[i].y = -1;
    }
    m_cells = 1;
}

bool Boats::gameOver()
{
    for(int i=0; i<boats; ++i)
    {
        if(m_boats[i].rotated)
        {
            for(int y1=m_boats[i].y; y1<brows && y1<m_boats[i].y+m_boats[i].cells; ++y1)
            {
                if(m_mycells[m_boats[i].x][y1].type != HIT)
                    return false;
            }
        }
        else
        {
            for(int x1=m_boats[i].x; x1<bcolumns && x1<m_boats[i].x+m_boats[i].cells; ++x1)
            {
                if(m_mycells[x1][m_boats[i].y].type != HIT)
                    return false;
            }
        }
    }
    return true;
}

bool Boats::boatDestroyed(int x, int y)
{
    for(int i=0; i<boats; ++i)
    {
        if(m_boats[i].constainsCell(x, y))
        {
            if(m_boats[i].rotated)
            {
                for(int y1=m_boats[i].y; y1<brows && y1<m_boats[i].y+m_boats[i].cells; ++y1)
                {
                    if(m_mycells[x][y1].type != HIT)
                        return false;
                }
            }
            else
            {
                for(int x1=m_boats[i].x; x1<bcolumns && x1<m_boats[i].x+m_boats[i].cells; ++x1)
                {
                    if(m_mycells[x1][y].type != HIT)
                        return false;
                }
            }
            return true;
        }
    }
    return false;
}

BoatsTabItem::BoatsTabItem(const QString &name, bool first, TabWidget *tabbook, int id, IrcEngine *engine, QWidget *parent)
    : dxTabItem(tabbook, id, parent), m_engine(engine), m_type(BOATS), m_name(name)
{
    m_gamecanvas = new Boats(first, this);
    m_gamecanvas->setFocus();

    m_newGame = new QPushButton(_("&New game"));
    m_newGame->setDisabled(true);
    QHBoxLayout *game = new QHBoxLayout;
    game->addWidget(m_newGame);
    game->addStretch(1);

    m_status = new QLabel(_("Place boats"));

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addWidget(m_gamecanvas, 1);
    mainLayout->addLayout(game);
    mainLayout->addWidget(m_status);
    setLayout(mainLayout);

    connect(m_gamecanvas, SIGNAL(fire(int,int)), this, SLOT(slotFire(int,int)));
    connect(m_gamecanvas, SIGNAL(statusText(QString)), this, SLOT(slotStatusText(QString)));
    connect(m_gamecanvas, SIGNAL(ready()), this, SLOT(slotReady()));
    connect(m_newGame, SIGNAL(clicked()), this, SLOT(slotNewGame()));
    if(m_engine)
    {
        connect(m_engine, SIGNAL(ircNick(QString,QString)), this, SLOT(slotIrcNick(QString,QString)));
        connect(m_engine, SIGNAL(ircQuit(QString,QString)), this, SLOT(slotIrcQuit(QString)));
        connect(m_engine, SIGNAL(irc401(QString)), this, SLOT(slotIrc401(QString)));
        connect(m_engine, SIGNAL(ircBoats(QString,QString,QString)), this, SLOT(slotIrcBoats(QString,QString,QString)));
        connect(m_engine, SIGNAL(ircBoatsReady(QString)), this, SLOT(slotIrcBoatsReady(QString)));
        connect(m_engine, SIGNAL(ircBoatsLeft(QString)), this, SLOT(slotIrcBoatsLeft(QString)));
        connect(m_engine, SIGNAL(ircBoatsDestroyed(QString)), this, SLOT(slotIrcBoatsDestoyed(QString)));
        connect(m_engine, SIGNAL(ircBoatsNewGame(QString)), this, SLOT(slotIrcBoatsNewGame(QString)));
        connect(m_engine, SIGNAL(ircBoatsWin(QString)), this, SLOT(slotIrcBoatsWin(QString)));
    }
}

BoatsTabItem::~BoatsTabItem()
{
}

void BoatsTabItem::leftGame()
{
    sendCtcp("BOATS LEFT");
}

bool BoatsTabItem::isSameServer(IrcEngine* engine)
{
    return engine == m_engine;
}

void BoatsTabItem::sendCtcp(const QString &msg)
{
    if(m_engine && m_engine->getConnected())
        m_engine->sendCtcp(m_name, msg);
}

void BoatsTabItem::slotIrcNick(QString nick, QString newnick)
{
    if(nick == m_name)
    {
        m_name = newnick;
        emit newname(m_name);
    }
}

void BoatsTabItem::slotIrcQuit(QString nick)
{
    if(nick == m_name)
    {
        m_gamecanvas->setPlaceBoats(false);
        m_gamecanvas->setReadyToPlay(false);
        m_gamecanvas->setOnFire(false);
        m_status->setText(_("%1 has quit").arg(nick));
    }
}

void BoatsTabItem::slotIrc401(QString nick)
{
    if(nick == m_name)
    {
        m_gamecanvas->setPlaceBoats(false);
        m_gamecanvas->setReadyToPlay(false);
        m_gamecanvas->setOnFire(false);
        m_status->setText(_("%1 isn't online").arg(nick));
    }
}

void BoatsTabItem::slotIrcBoats(QString nick, QString position, QString status)
{
    if(nick == m_name)
    {
        //position
        if(position.length()!=2)
            return;
        if(position[0].toLatin1()<65 || position[0].toLatin1()>74)
            return;
        if(position[1].toLatin1()<48 || position[1].toLatin1()>57)
            return;
        int x = position[0].toLatin1()-65;
        int y = position[1].toLatin1()-48;
        //redraw cell
        if(status == "FIRE")
        {
            if(m_gamecanvas->getCellType(x, y, true) == BOAT)
            {
                m_gamecanvas->setCellType(x, y, true, HIT);
                sendCtcp(QString("BOATS %1 HIT").arg(position));
                if(m_gamecanvas->boatDestroyed(x, y))
                    sendCtcp("BOATS DESTROYED");
                if(m_gamecanvas->gameOver())
                {
                    m_gamecanvas->setReadyToPlay(false);
                    m_gamecanvas->setOnFire(false);
                    m_status->setText(_("You LOST."));
                    m_newGame->setEnabled(true);
                    sendCtcp("BOATS WIN");
                    m_gamecanvas->update();
                    return;
                }
            }
            else
            {
                m_gamecanvas->setCellType(x, y, true, WATER);
                sendCtcp(QString("BOATS %1 FAIL").arg(position));
            }
            m_gamecanvas->update();
            m_status->setText(_("Adversary fired. You fire NOW!"));
            m_gamecanvas->setOnFire(true);
            return;
        }
        if(status == "HIT")
        {
            m_gamecanvas->setCellType(x, y, false, HIT);
            m_gamecanvas->update();
            m_status->setText(_("Waiting for adversary fire."));
            return;
        }
        if(status == "FAIL")
        {
            m_gamecanvas->setCellType(x, y, false, WATER);
            m_gamecanvas->update();
            m_status->setText(_("Waiting for adversary fire."));
            return;
        }
    }
}

void BoatsTabItem::slotIrcBoatsReady(QString nick)
{
    if(nick == m_name)
    {
        m_gamecanvas->setReadyToPlay(true);
        if(m_gamecanvas->first())
        {
            m_gamecanvas->setOnFire(true);
            if(!m_gamecanvas->placeBoats()) m_status->setText(_("Fire NOW!"));
        }
        else
        {
            m_gamecanvas->setOnFire(false);
            if(!m_gamecanvas->placeBoats()) m_status->setText(_("Waiting for adversary fire."));
        }
    }
}

void BoatsTabItem::slotIrcBoatsLeft(QString nick)
{
    if(nick == m_name)
    {
        m_gamecanvas->setPlaceBoats(false);
        m_gamecanvas->setReadyToPlay(false);
        m_gamecanvas->setOnFire(false);
        m_status->setText(_("Adversary left game."));
    }
}

void BoatsTabItem::slotIrcBoatsDestoyed(QString nick)
{
    if(nick == m_name)
    {
        m_status->setText(_("Boat was destroyed. Waiting for adversary fire."));
    }
}

void BoatsTabItem::slotIrcBoatsNewGame(QString nick)
{
    if(nick == m_name)
    {
        m_gamecanvas->clearCellsAndBoats();
        m_gamecanvas->setPlaceBoats(true);
        m_gamecanvas->setReadyToPlay(false);
        m_gamecanvas->setOnFire(false);
        m_status->setText(_("Place boats"));
        m_newGame->setDisabled(true);
    }
}

void BoatsTabItem::slotIrcBoatsWin(QString nick)
{
    if(nick == m_name)
    {
        m_gamecanvas->setReadyToPlay(false);
        m_gamecanvas->setOnFire(false);
        m_status->setText(_("You WIN."));
        m_newGame->setEnabled(true);
    }
}

void BoatsTabItem::slotFire(int x, int y)
{
    if(m_engine->getConnected())
    {
        sendCtcp(QString("BOATS %1%2 FIRE").arg(static_cast<char>(x+65)).arg(static_cast<char>(y+48)));
        m_status->setText(_("Waiting for response of fire"));
    }
}

void BoatsTabItem::slotStatusText(QString text)
{
    m_status->setText(text);
}

void BoatsTabItem::slotReady()
{
    if(m_engine->getConnected())
    {
        sendCtcp("BOATS READY");
        if(m_gamecanvas->readyToPlay())
        {
            if(m_gamecanvas->first())
            {
                m_status->setText(_("Fire NOW!"));
            }
            else
            {
                m_status->setText(_("Waiting for adversary fire."));
            }
        }
    }
}

void BoatsTabItem::slotNewGame()
{
    if(m_engine->getConnected())
    {
        sendCtcp("BOATS NEWGAME");
        m_gamecanvas->clearCellsAndBoats();
        m_gamecanvas->setPlaceBoats(true);
        m_gamecanvas->setReadyToPlay(false);
        m_gamecanvas->setOnFire(false);
        m_status->setText(_("Place boats"));
        m_newGame->setDisabled(true);
    }
}
