/*
 *      dcctabitem.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef DCCTABITEM_H
#define DCCTABITEM_H

#include <fstream>
#include "dxtabitem.h"
#include "widgets/linespell.h"
#ifndef DXSOCKET_H
#include "engine/dxsocket.h"
#endif
#include "engine/ircengine.h"

class DccTabItem: public dxTabItem
{
    Q_OBJECT
    friend class dxirc;
public:
    DccTabItem(TabWidget *tab, const QString &mynick, const QString &nick, const QString &address="0", int portD=0, int portH=0, bool listen=false, int id=0, IrcEngine *engine=NULL, QWidget *parent=NULL);
    virtual ~DccTabItem();
    enum Event{
        DCCMSG,
        DCCACTION,
        CONNECT,
        DISCONNECT,
        DCCERROR
    };

    void clearChat();
    QString getServerName() { return m_address; }
    int getServerPort() { return m_portD; }
    QString getNickName() { return m_mynick; }
    QString getRealServerName() { return m_address; }
    TYPE getType() { return DCC; }
    void preferencesUpdated();
    void setCommandFocus();
    void showSpellComboUpdated();
    void makeLastRowVisible();
    QString getSpellLang();
    bool getConnected() { return m_socket->isConnected(); }
    void disconnectDcc() { m_socket->disconnect(); }
    QString getTabName() { return m_nick; }
signals:
    void highlight(TYPE);
    void newmsg(TYPE);
    void ownMsg(QString);
    void command(QString);
    void luaRequest(LuaRequest);
    void newlog(QString logpath);
    void newname(QString);
    void newtetris();
protected:
    bool eventFilter(QObject *obj, QEvent *ev);
    void appendText(QString msg, bool highlight, bool logLine=true);
    void appendStyledText(QString text, int style, bool highlight, bool disableStrip=false, bool logLine=true);
    bool processLine(const QString &);
    void setHasAllCommand(bool);
    void setHasOwnMsg(bool);
private:
    dxSocket *m_server;
    QProcess *m_pipe;
    QTextBrowser *m_text;
    LineSpell *m_commandline;
    int m_currentPosition, m_historyMax, m_dccTimeout;
    bool m_sendPipe, m_listen;
    bool m_scriptHasAll, m_scriptHasOwnMsg;
    QList<QString> m_commandsHistory, m_pipeStrings;
    std::ofstream *m_logstream;
    QComboBox *m_spellLangs;
    QString m_mynick, m_nick, m_address, m_receiveRest, m_dccIP;
    int m_portD, m_portH;
    dxSocket *m_socket;
    IrcEngine *m_engine;
    QTimer *m_pipeTimer;
    QTimer *m_closeTimer;

    void appendIrcText(QString msg, QString time, bool disableStrip=false, bool logLine=true);
    void appendIrcStyledText(QString styled, int stylenum, QString time, bool disableStrip=false, bool logLine=true);
    void appendIrcNickText(QString nick, QString msg, QString time, bool logLine=true);
    void appendMyMsg(QString msg, QString time);
    void startLogging();
    void stopLogging();
    void logLine(const QString &line, const QString &time);
    bool processCommand(const QString &);
    bool showHelp(QString command);
    QStringList cutText(QString, int);
    bool highlightNeeded(const QString &);
    bool sendLine(const QString &line);
    void loadLogLines();
    void listLogFiles();
    void completeHelpCommand();
    void start();
    void writeEvent(const QString &msg, Event event);
private slots:
    void slotProcessCommandline();
    void slotOnPipe();
    void slotPipeTimeout();
    void slotCloseTimeout();
    void slotIrcNick(QString nick, QString newnick);
    void slotCanRead();
    void slotConnected();
    void slotDisconnected();
    void slotError(QString error);
    void slotStartAccept(int port);
    void slotSomeConnected();
};

#endif // DCCCTABITEM_H
